@@ -13,11 +13,15 @@
 #include "logo.h"
 #include "soppalusikka.h"
 
-#if defined(APIVERSNUM) && APIVERSNUM < 10718
-#error "VDR-1.7.18 API version or greater is required!"
+#if defined(APIVERSNUM) && APIVERSNUM < 10730
+#error "VDR-1.7.30 API version or greater is required!"
 #endif
 
-static const char VERSION[]     = "1.7.4";
+#ifndef GITVERSION
+#define GITVERSION ""
+#endif
+
+static const char VERSION[]     = "1.7.8" GITVERSION;
 static const char DESCRIPTION[] = trNOOP("Soppalusikka skin");
 
 class cPluginSkinSoppalusikka : public cPlugin {
@@ -112,7 +116,7 @@ bool cPluginSkinSoppalusikka::Start(void)
   debug("cPluginSkinSoppalusikka::Start()");
   // set logo directory
   if (!islogodirset) {
-     SoppalusikkaConfig.SetLogoDir(cPlugin::ConfigDirectory(PLUGIN_NAME_I18N));
+     SoppalusikkaConfig.SetLogoDir(cPlugin::ResourceDirectory(PLUGIN_NAME_I18N));
      islogodirset = true;
      }
   // resize logo cache
