@@ -19,6 +19,7 @@
 #include <vdr/osd.h>
 #include <vdr/themes.h>
 #include <vdr/plugin.h>
+#include <vdr/videodir.h>
 
 #define GetSymbol(id)     SoppalusikkaSymbolCache.Get(id)
 #define TinyGap           SoppalusikkaSymbolCache.GetGapTiny()
@@ -299,11 +300,11 @@ void cSkinSoppalusikkaDisplayChannel::DrawSignalArea(void)
   // draw signal strength
   x = (xs1 - xs0 - 1) * (s < 0 ? 0 : s) / 100;
   osd->DrawRectangle(xs0, ys0, xs1 - 1, ys0 + Gap, Theme.Color(clrChannelSymbolInactive));
-  osd->DrawRectangle(xs0, ys0, xs0 + x, ys0 + Gap, Theme.Color(clrChannelSymbolActive));
+  osd->DrawRectangle(xs0, ys0, xs0 + x - 1, ys0 + Gap, Theme.Color(clrChannelSymbolActive));
   // draw signal quality
   x = (xs1 - xs0 - 1) * (q < 0 ? 0 : q) / 100;
   osd->DrawRectangle(xs0, ys0 + 2 * Gap, xs1 - 1, ys0 + 3 * Gap, Theme.Color(clrChannelSymbolInactive));
-  osd->DrawRectangle(xs0, ys0 + 2 * Gap, xs0 + x, ys0 + 3 * Gap, Theme.Color(clrChannelSymbolActive));
+  osd->DrawRectangle(xs0, ys0 + 2 * Gap, xs0 + x - 1, ys0 + 3 * Gap, Theme.Color(clrChannelSymbolActive));
   // update timer
   lastSignalUpdate.Set(SIGNALUPDATETIMEOUT_MS);
 }
@@ -540,8 +541,11 @@ private:
   int x0, x1, x2, x3, x4, x5;
   int y0, y1, y2, y3, y4, y5, y6, y7, y8;
   int lineHeight;
+  int lastDiskUsageState;
   bool lastRec;
   cString lastDate;
+  cString title;
+  void DrawTitle();
   void DrawScrollbar(int Total, int Offset, int Shown, int Top, int Height, bool CanScrollUp, bool CanScrollDown);
   void SetTextScrollbar(void);
 public:
@@ -567,6 +571,7 @@ cSkinSoppalusikkaDisplayMenu::cSkinSoppalusikkaDisplayMenu(void)
 {
   const cFont *font = cFont::GetFont(fontOsd);
   lineHeight = font->Height();
+  lastDiskUsageState = -1;
   lastRec = false;
   x0 = 0;
   x1 = x0 + Gap;
@@ -684,16 +689,28 @@ void cSkinSoppalusikkaDisplayMenu::Clear(void)
   osd->DrawRectangle(x0, y2, x5 - 1, y6 - 1, Theme.Color(clrBackground));
 }
 
-void cSkinSoppalusikkaDisplayMenu::SetTitle(const char *Title)
+void cSkinSoppalusikkaDisplayMenu::DrawTitle()
 {
   const cFont *font = cFont::GetFont(fontSml);
   int w = Gap + font->Width("Wmm 07.07 07:07") + (cRecordControls::Active() ?  GetSymbol(SYMBOL_RECORDING).Width() : 0);
   // draw title
-  osd->DrawText(x2, y0, Title, Theme.Color(clrMenuTitleFg), Theme.Color(clrMenuTitleBg), cFont::GetFont(fontSml), x3 - x2 - w, y2 - y0);
+  if ((MenuCategory() == mcMain) || (MenuCategory() == mcRecording))
+     osd->DrawText(x2, y0, cString::sprintf("%s  -  %s", *title, *cVideoDiskUsage::String()), Theme.Color(clrMenuTitleFg), Theme.Color(clrMenuTitleBg), cFont::GetFont(fontSml), x3 - x2 - w, y2 - y0);
+  else
+     osd->DrawText(x2, y0, title, Theme.Color(clrMenuTitleFg), Theme.Color(clrMenuTitleBg), cFont::GetFont(fontSml), x3 - x2 - w, y2 - y0);
+}
+
+void cSkinSoppalusikkaDisplayMenu::SetTitle(const char *Title)
+{
+  title = Title;
+  DrawTitle();
 }
 
 void cSkinSoppalusikkaDisplayMenu::SetButtons(const char *Red, const char *Green, const char *Yellow, const char *Blue)
 {
+  const char *lutText[] = { Red, Green, Yellow, Blue };
+  tColor lutFg[] = { clrButtonRedFg, clrButtonGreenFg, clrButtonYellowFg, clrButtonBlueFg };
+  tColor lutBg[] = { clrButtonRedBg, clrButtonGreenBg, clrButtonYellowBg, clrButtonBlueBg };
   const cFont *font = cFont::GetFont(fontSml);
   int t0 = x2;
   int t4 = x3;
@@ -701,10 +718,10 @@ void cSkinSoppalusikkaDisplayMenu::SetButtons(const char *Red, const char *Green
   int t1 = t0 + (t2 - t0) / 2;
   int t3 = t2 + (t4 - t2) / 2;
   // draw color buttons
-  osd->DrawText(t0, y6, Red,    Theme.Color(clrButtonRedFg),    Theme.Color(clrButtonRedBg),    font, t1 - t0, y8 - y6, taCenter);
-  osd->DrawText(t1, y6, Green,  Theme.Color(clrButtonGreenFg),  Theme.Color(clrButtonGreenBg),  font, t2 - t1, y8 - y6, taCenter);
-  osd->DrawText(t2, y6, Yellow, Theme.Color(clrButtonYellowFg), Theme.Color(clrButtonYellowBg), font, t3 - t2, y8 - y6, taCenter);
-  osd->DrawText(t3, y6, Blue,   Theme.Color(clrButtonBlueFg),   Theme.Color(clrButtonBlueBg),   font, t4 - t3, y8 - y6, taCenter);
+  osd->DrawText(t0, y6, lutText[Setup.ColorKey0], Theme.Color(lutFg[Setup.ColorKey0]), Theme.Color(lutBg[Setup.ColorKey0]), font, t1 - t0, y8 - y6, taCenter);
+  osd->DrawText(t1, y6, lutText[Setup.ColorKey1], Theme.Color(lutFg[Setup.ColorKey1]), Theme.Color(lutBg[Setup.ColorKey1]), font, t2 - t1, y8 - y6, taCenter);
+  osd->DrawText(t2, y6, lutText[Setup.ColorKey2], Theme.Color(lutFg[Setup.ColorKey2]), Theme.Color(lutBg[Setup.ColorKey2]), font, t3 - t2, y8 - y6, taCenter);
+  osd->DrawText(t3, y6, lutText[Setup.ColorKey3], Theme.Color(lutFg[Setup.ColorKey3]), Theme.Color(lutBg[Setup.ColorKey3]), font, t4 - t3, y8 - y6, taCenter);
 }
 
 void cSkinSoppalusikkaDisplayMenu::SetMessage(eMessageType Type, const char *Text)
@@ -749,7 +766,7 @@ void cSkinSoppalusikkaDisplayMenu::SetItem(const char *Text, int Index, bool Cur
          bool isprogressbar = false;
          int now = 0, total = 0;
          // check if event info symbol: "tTV*" "R"
-         if (SoppalusikkaConfig.showsymbols) {
+         if (SoppalusikkaConfig.showsymbols /*&& (MenuCategory() == mcSchedule)*/) {
             // check if event info characters
             if (strlen(s) == 3 && ischaracter(s[0], " tTR") && ischaracter(s[1], " V") && ischaracter(s[2], " *")) {
                // update status
@@ -757,7 +774,7 @@ void cSkinSoppalusikkaDisplayMenu::SetItem(const char *Text, int Index, bool Cur
                }
             }
          // check if new recording: "01.01.06*", "10:10*"
-         if (!iseventinfo && SoppalusikkaConfig.showsymbols &&
+         if (!iseventinfo && SoppalusikkaConfig.showsymbols /*&& (MenuCategory() == mcRecording)*/ &&
             (strlen(s) == 6 && s[5] == '*' && s[2] == ':' && isdigit(*s) && isdigit(*(s + 1)) && isdigit(*(s + 3)) && isdigit(*(s + 4))) ||
             (strlen(s) == 9 && s[8] == '*' && s[5] == '.' && s[2] == '.' && isdigit(*s) && isdigit(*(s + 1)) && isdigit(*(s + 3)) && isdigit(*(s + 4)) && isdigit(*(s + 6)) && isdigit(*(s + 7)))) {
             // update status
@@ -768,7 +785,7 @@ void cSkinSoppalusikkaDisplayMenu::SetItem(const char *Text, int Index, bool Cur
             buffer[strlen(s) - 1] = '\0';
             }
          // check if progress bar: "[|||||||   ]"
-         if (!iseventinfo && !isnewrecording && SoppalusikkaConfig.showprogressbar &&
+         if (!iseventinfo && !isnewrecording && SoppalusikkaConfig.showprogressbar /*&& (MenuCategory() == mcSchedule)*/ &&
             (strlen(s) > 5 && s[0] == '[' && s[strlen(s) - 1] == ']')) {
             const char *p = s + 1;
             // update status
@@ -1018,11 +1035,11 @@ void cSkinSoppalusikkaDisplayMenu::SetRecording(const cRecording *Recording)
   cTextScroller ts;
   char t[32];
   // draw recording date string
-  snprintf(t, sizeof(t), "%s  %s", *DateString(Recording->start), *TimeString(Recording->start));
+  snprintf(t, sizeof(t), "%s  %s", *DateString(Recording->Start()), *TimeString(Recording->Start()));
   ts.Set(osd, x2, y, x3 - x2, y5 - y, t, font, Theme.Color(clrMenuEventTime), Theme.Color(clrBackground));
   y += ts.Height();
   // draw additional information
-  cString info = cString::sprintf("%s: %d %s: %d", tr("Priority"), Recording->priority, tr("Lifetime"), Recording->lifetime);
+  cString info = cString::sprintf("%s: %d %s: %d", tr("Priority"), Recording->Priority(), tr("Lifetime"), Recording->Lifetime());
   if (SoppalusikkaConfig.showauxinfo && Info->Aux()) {
      char *aux = strdup(Info->Aux());
      info = cString::sprintf("%s\n%s: %s", *info, tr("Auxiliary information"), parseaux(aux));
@@ -1139,6 +1156,8 @@ void cSkinSoppalusikkaDisplayMenu::Flush(void)
   const cFont *font = cFont::GetFont(fontSml);
   int w = font->Width("Wmm 07.07 07:07");
   bool rec = cRecordControls::Active();
+  if (cVideoDiskUsage::HasChanged(lastDiskUsageState))
+     DrawTitle();
   if (lastRec != rec) {
      int xl = x3 - w - GetSymbol(SYMBOL_RECORDING).Width();
      // draw safety margin
@@ -1169,6 +1188,7 @@ private:
 public:
   cSkinSoppalusikkaDisplayReplay(bool ModeOnly);
   virtual ~cSkinSoppalusikkaDisplayReplay();
+  virtual void SetRecording(const cRecording *Recording);
   virtual void SetTitle(const char *Title);
   virtual void SetMode(bool Play, bool Forward, int Speed);
   virtual void SetProgress(int Current, int Total);
@@ -1259,6 +1279,11 @@ cSkinSoppalusikkaDisplayReplay::~cSkinSoppalusikkaDisplayReplay()
   delete osd;
 }
 
+void cSkinSoppalusikkaDisplayReplay::SetRecording(const cRecording *Recording)
+{
+  SetTitle(Recording->Title());
+}
+
 void cSkinSoppalusikkaDisplayReplay::SetTitle(const char *Title)
 {
   // draw titlebar
@@ -1401,7 +1426,7 @@ void cSkinSoppalusikkaDisplayReplay::SetMode(bool Play, bool Forward, int Speed)
   else {
      // draw inactive symbol
      xs -= GetSymbol(SYMBOL_FREW).Width() + Gap;
-     osd->DrawBitmap(xs, y3 + (y5 - y3 - GetSymbol(SYMBOL_FREW).Height()) / 2, GetSymbol(SYMBOL_TELETEXT), Theme.Color(clrReplaySymbolInactive), Theme.Color(clrBackground));
+     osd->DrawBitmap(xs, y3 + (y5 - y3 - GetSymbol(SYMBOL_FREW).Height()) / 2, GetSymbol(SYMBOL_FREW), Theme.Color(clrReplaySymbolInactive), Theme.Color(clrBackground));
      }
 }
 
