@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: channels.c 2.19 2011/08/26 12:44:21 kls Exp $
+ * $Id: channels.c 2.24 2012/07/14 12:34:47 kls Exp $
  */
 
 #include "channels.h"
@@ -112,10 +112,34 @@ cChannel& cChannel::operator= (const cChannel &Channel)
   provider = strcpyrealloc(provider, Channel.provider);
   portalName = strcpyrealloc(portalName, Channel.portalName);
   memcpy(&__BeginData__, &Channel.__BeginData__, (char *)&Channel.__EndData__ - (char *)&Channel.__BeginData__);
+  nameSource = NULL; // these will be recalculated automatically
+  shortNameSource = NULL;
   parameters = Channel.parameters;
   return *this;
 }
 
+const char *cChannel::Name(void) const
+{
+  if (Setup.ShowChannelNamesWithSource && !groupSep) {
+     if (isempty(nameSource))
+        nameSource = cString::sprintf("%s (%c)", name, cSource::ToChar(source));
+     return nameSource;
+     }
+  return name;
+}
+
+const char *cChannel::ShortName(bool OrName) const
+{
+  if (OrName && isempty(shortName))
+     return Name();
+  if (Setup.ShowChannelNamesWithSource && !groupSep) {
+     if (isempty(shortNameSource))
+        shortNameSource = cString::sprintf("%s (%c)", shortName, cSource::ToChar(source));
+     return shortNameSource;
+     }
+  return shortName;
+}
+
 int cChannel::Transponder(int Frequency, char Polarization)
 {
   // some satellites have transponders at the same frequency, just with different polarization:
@@ -193,6 +217,8 @@ bool cChannel::SetTransponderData(int Source, int Frequency, int Srate, const ch
      srate = Srate;
      parameters = Parameters;
      schedule = NULL;
+     nameSource = NULL;
+     shortNameSource = NULL;
      if (Number() && !Quiet) {
         dsyslog("changing transponder data of channel %d from %s to %s", Number(), *OldTransponderData, *TransponderDataToString());
         modification |= CHANNELMOD_TRANSP;
@@ -233,10 +259,14 @@ void cChannel::SetName(const char *Name, const char *ShortName, const char *Prov
            modification |= CHANNELMOD_NAME;
            Channels.SetModified();
            }
-        if (nn)
+        if (nn) {
            name = strcpyrealloc(name, Name);
-        if (ns)
+           nameSource = NULL;
+           }
+        if (ns) {
            shortName = strcpyrealloc(shortName, ShortName);
+           shortNameSource = NULL;
+           }
         if (np)
            provider = strcpyrealloc(provider, Provider);
         }
@@ -381,9 +411,9 @@ void cChannel::SetSubtitlingDescriptors(uchar *SubtitlingTypes, uint16_t *Compos
 void cChannel::SetTeletextSubtitlePages(tTeletextSubtitlePage pages[], int numberOfPages)
 {
   int mod = CHANNELMOD_NONE;
-  if (totalTtxtSubtitlePages != numberOfPages)
+  if (totalTtxtSubtitlePages != (fixedTtxtSubtitlePages + numberOfPages))
      mod |= CHANNELMOD_PIDS;
-  totalTtxtSubtitlePages = 0;
+  totalTtxtSubtitlePages = fixedTtxtSubtitlePages;
   for (int i = 0; (i < numberOfPages) && (totalTtxtSubtitlePages < MAXTXTPAGES); i++) {
       if (teletextSubtitlePages[totalTtxtSubtitlePages].ttxtMagazine != pages[i].ttxtMagazine ||
           teletextSubtitlePages[totalTtxtSubtitlePages].ttxtPage != pages[i].ttxtPage ||
@@ -526,10 +556,17 @@ cString cChannel::ToText(const cChannel *Channel)
         q += IntArrayToString(q, Channel->dpids, 10, Channel->dlangs, Channel->dtypes);
         }
      *q = 0;
-     const int TBufferSize = MAXSPIDS * (5 + 1 + MAXLANGCODE2) + 10; // 5 digits plus delimiting ',' or ';' plus optional '=cod+cod', +10: paranoia and tpid
+     const int TBufferSize = (MAXTXTPAGES * MAXSPIDS) * (5 + 1 + MAXLANGCODE2) + 10; // 5 digits plus delimiting ',' or ';' plus optional '=cod+cod', +10: paranoia and tpid
      char tpidbuf[TBufferSize];
      q = tpidbuf;
      q += snprintf(q, sizeof(tpidbuf), "%d", Channel->tpid);
+     if (Channel->fixedTtxtSubtitlePages > 0) {
+        *q++ = '+';
+        for (int i = 0; i < Channel->fixedTtxtSubtitlePages; ++i) {
+            tTeletextSubtitlePage page = Channel->teletextSubtitlePages[i];
+            q += snprintf(q, sizeof(tpidbuf) - (q - tpidbuf), "%d=%s", page.PageNumber(), page.ttxtLanguage);
+            }
+        }
      if (Channel->spids[0]) {
         *q++ = ';';
         q += IntArrayToString(q, Channel->spids, 10, Channel->slangs);
@@ -639,7 +676,8 @@ bool cChannel::Parse(const char *s)
                        }
                     else
                        *alangs[NumApids] = 0;
-                    apids[NumApids++] = strtol(q, NULL, 10);
+                    if ((apids[NumApids] = strtol(q, NULL, 10)) != 0)
+                       NumApids++;
                     }
                  else
                     esyslog("ERROR: too many APIDs!"); // no need to set ok to 'false'
@@ -667,7 +705,8 @@ bool cChannel::Parse(const char *s)
                           }
                        else
                           *dlangs[NumDpids] = 0;
-                       dpids[NumDpids++] = strtol(q, NULL, 10);
+                       if ((dpids[NumDpids] = strtol(q, NULL, 10)) != 0)
+                          NumDpids++;
                        }
                     else
                        esyslog("ERROR: too many DPIDs!"); // no need to set ok to 'false'
@@ -698,6 +737,32 @@ bool cChannel::Parse(const char *s)
                     }
               spids[NumSpids] = 0;
               }
+           fixedTtxtSubtitlePages = 0;
+           if ((p = strchr(tpidbuf, '+')) != NULL) {
+              *p++ = 0;
+              char *q;
+              char *strtok_next;
+              while ((q = strtok_r(p, ",", &strtok_next)) != NULL) {
+                    if (fixedTtxtSubtitlePages < MAXTXTPAGES) {
+                       int page;
+                       char *l = strchr(q, '=');
+                       if (l)
+                          *l++ = 0;
+                       if (sscanf(q, "%d", &page) == 1) {
+                          teletextSubtitlePages[fixedTtxtSubtitlePages] = tTeletextSubtitlePage(page);
+                          if (l)
+                             strn0cpy(teletextSubtitlePages[fixedTtxtSubtitlePages].ttxtLanguage, l, MAXLANGCODE2);
+                          fixedTtxtSubtitlePages++;
+                          }
+                       else
+                          esyslog("ERROR: invalid Teletext page!"); // no need to set ok to 'false'
+                       }
+                    else
+                       esyslog("ERROR: too many Teletext pages!"); // no need to set ok to 'false'
+                    p = NULL;
+                    }
+              totalTtxtSubtitlePages = fixedTtxtSubtitlePages;
+              }
            if (sscanf(tpidbuf, "%d", &tpid) != 1)
               return false;
            if (caidbuf) {
@@ -739,6 +804,8 @@ bool cChannel::Parse(const char *s)
         free(tpidbuf);
         free(caidbuf);
         free(namebuf);
+        nameSource = NULL;
+        shortNameSource = NULL;
         if (!GetChannelID().Valid()) {
            esyslog("ERROR: channel data results in invalid ID!");
            return false;
@@ -778,6 +845,8 @@ cChannels Channels;
 cChannels::cChannels(void)
 {
   maxNumber = 0;
+  maxChannelNameLength = 0;
+  maxShortChannelNameLength = 0;
   modified = CHANNELSMOD_NONE;
 }
 
@@ -955,9 +1024,32 @@ bool cChannels::SwitchTo(int Number)
   return channel && cDevice::PrimaryDevice()->SwitchChannel(channel, true);
 }
 
+int cChannels::MaxChannelNameLength(void)
+{
+  if (!maxChannelNameLength) {
+     for (cChannel *channel = First(); channel; channel = Next(channel)) {
+         if (!channel->GroupSep())
+            maxChannelNameLength = max(Utf8StrLen(channel->Name()), maxChannelNameLength);
+         }
+     }
+  return maxChannelNameLength;
+}
+
+int cChannels::MaxShortChannelNameLength(void)
+{
+  if (!maxShortChannelNameLength) {
+     for (cChannel *channel = First(); channel; channel = Next(channel)) {
+         if (!channel->GroupSep())
+            maxShortChannelNameLength = max(Utf8StrLen(channel->ShortName(true)), maxShortChannelNameLength);
+         }
+     }
+  return maxShortChannelNameLength;
+}
+
 void cChannels::SetModified(bool ByUser)
 {
   modified = ByUser ? CHANNELSMOD_USER : !modified ? CHANNELSMOD_AUTO : modified;
+  maxChannelNameLength = maxShortChannelNameLength = 0;
 }
 
 int cChannels::Modified(void)
