@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: channels.h 2.14 2011/08/06 09:56:13 kls Exp $
+ * $Id: channels.h 2.16 2012/06/17 11:21:33 kls Exp $
  */
 
 #ifndef __CHANNELS_H
@@ -126,6 +126,7 @@ private:
   uint16_t compositionPageIds[MAXSPIDS];
   uint16_t ancillaryPageIds[MAXSPIDS];
   int tpid;
+  int fixedTtxtSubtitlePages;
   int totalTtxtSubtitlePages;
   tTeletextSubtitlePage teletextSubtitlePages[MAXTXTPAGES];
   int caids[MAXCAIDS + 1]; // list is zero-terminated
@@ -134,8 +135,11 @@ private:
   int sid;
   int rid;
   int number;    // Sequence number assigned on load
+  int lcn;       // Logical channel number assigned by data stream
   bool groupSep;
   int __EndData__;
+  mutable cString nameSource;
+  mutable cString shortNameSource;
   cString parameters;
   int modification;
   mutable const cSchedule *schedule;
@@ -150,8 +154,8 @@ public:
   cString ToText(void) const;
   bool Parse(const char *s);
   bool Save(FILE *f);
-  const char *Name(void) const { return name; }
-  const char *ShortName(bool OrName = false) const { return (OrName && isempty(shortName)) ? name : shortName; }
+  const char *Name(void) const;
+  const char *ShortName(bool OrName = false) const;
   const char *Provider(void) const { return provider; }
   const char *PortalName(void) const { return portalName; }
   int Frequency(void) const { return frequency; } ///< Returns the actual frequency, as given in 'channels.conf'
@@ -187,6 +191,8 @@ public:
   int Rid(void) const { return rid; }
   int Number(void) const { return number; }
   void SetNumber(int Number) { number = Number; }
+  int Lcn(void) const { return lcn; }
+  void SetLCN(int Lcn) { lcn = Lcn; }
   bool GroupSep(void) const { return groupSep; }
   const char *Parameters(void) const { return parameters; }
   const cLinkChannels* LinkChannels(void) const { return linkChannels; }
@@ -216,6 +222,8 @@ public:
 class cChannels : public cRwLock, public cConfig<cChannel> {
 private:
   int maxNumber;
+  int maxChannelNameLength;
+  int maxShortChannelNameLength;
   int modified;
   int beingEdited;
   cHash<cChannel> channelsHashSid;
@@ -240,6 +248,8 @@ public:
   bool HasUniqueChannelID(cChannel *NewChannel, cChannel *OldChannel = NULL);
   bool SwitchTo(int Number);
   int MaxNumber(void) { return maxNumber; }
+  int MaxChannelNameLength(void);
+  int MaxShortChannelNameLength(void);
   void SetModified(bool ByUser = false);
   int Modified(void);
       ///< Returns 0 if no channels have been modified, 1 if an automatic
