@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: ci.c 2.7 2011/05/21 15:21:33 kls Exp $
+ * $Id: ci.c 2.9 2012/05/29 11:13:40 kls Exp $
  */
 
 #include "ci.h"
@@ -1715,6 +1715,7 @@ bool cCamSlot::Reset(void)
      if (ciAdapter->Reset(slotIndex)) {
         resetTime = time(NULL);
         dbgprotocol("ok.\n");
+        lastModuleStatus = msReset;
         return true;
         }
      dbgprotocol("failed!\n");
@@ -1841,7 +1842,7 @@ const int *cCamSlot::GetCaSystemIds(void)
 int cCamSlot::Priority(void)
 {
   cDevice *d = Device();
-  return d ? d->Priority() : -1;
+  return d ? d->Priority() : IDLEPRIORITY;
 }
 
 bool cCamSlot::ProvidesCa(const int *CaSystemIds)
