@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: device.h 2.27 2011/08/26 12:52:29 kls Exp $
+ * $Id: device.h 2.41 2012/08/26 13:25:44 kls Exp $
  */
 
 #ifndef __DEVICE_H
@@ -30,6 +30,7 @@
 #define MAXRECEIVERS       16 // the maximum number of receivers per device
 #define MAXVOLUME         255
 #define VOLUMEDELTA         5 // used to increase/decrease the volume
+#define MAXOCCUPIEDTIMEOUT 99 // max. time (in seconds) a device may be occupied
 
 enum eSetChannelResult { scrOk, scrNotAvailable, scrNoTransfer, scrFailed };
 
@@ -108,7 +109,6 @@ private:
   static int useDevice;
   static cDevice *device[MAXDEVICES];
   static cDevice *primaryDevice;
-  static cDevice *avoidDevice;
 public:
   static int NumDevices(void) { return numDevices; }
          ///< Returns the total number of devices.
@@ -138,7 +138,7 @@ public:
          ///< Gets the device with the given Index.
          ///< \param Index must be in the range 0..numDevices-1.
          ///< \return A pointer to the device, or NULL if the Index was invalid.
-  static cDevice *GetDevice(const cChannel *Channel, int Priority, bool LiveView);
+  static cDevice *GetDevice(const cChannel *Channel, int Priority, bool LiveView, bool Query = false);
          ///< Returns a device that is able to receive the given Channel at the
          ///< given Priority, with the least impact on active recordings and
          ///< live viewing. The LiveView parameter tells whether the device will
@@ -152,10 +152,16 @@ public:
          ///< after detaching any receivers because the channel can't be decrypted,
          ///< this device/CAM combination will be skipped in the next call to
          ///< GetDevice().
+         ///< If Query is true, no actual CAM assignments or receiver detachments will
+         ///< be done, so that this function can be called without any side effects
+         ///< in order to just determine whether a device is available for the given
+         ///< Channel.
          ///< See also ProvidesChannel().
-  static void SetAvoidDevice(cDevice *Device) { avoidDevice = Device; }
-         ///< Sets the given Device to be temporarily avoided in the next call to
-         ///< GetDevice(const cChannel, int, bool).
+  static cDevice *GetDeviceForTransponder(const cChannel *Channel, int Priority);
+         ///< Returns a device that is not currently "occupied" and can be tuned to
+         ///< the transponder of the given Channel, without disturbing any receiver
+         ///< at priorities higher or equal to Priority.
+         ///< If no such device is currently available, NULL will be returned.
   static void Shutdown(void);
          ///< Closes down all devices.
          ///< Must be called at the end of the program.
@@ -195,6 +201,15 @@ public:
          ///< Returns the card index of this device (0 ... MAXDEVICES - 1).
   int DeviceNumber(void) const;
          ///< Returns the number of this device (0 ... numDevices).
+  virtual cString DeviceType(void) const;
+         ///< Returns a string identifying the type of this device (like "DVB-S").
+         ///< If this device can receive different delivery systems, the returned
+         ///< string shall be that of the currently used system.
+         ///< The length of the returned string should not exceed 6 characters.
+         ///< The default implementation returns an empty string.
+  virtual cString DeviceName(void) const;
+         ///< Returns a string identifying the name of this device.
+         ///< The default implementation returns an empty string.
   virtual bool HasDecoder(void) const;
          ///< Tells whether this device has an MPEG decoder.
   virtual bool AvoidRecording(void) const { return false; }
@@ -220,6 +235,8 @@ public:
 
 // Channel facilities
 
+private:
+  time_t occupiedTimeout;
 protected:
   static int currentChannel;
 public:
@@ -232,12 +249,11 @@ public:
   virtual bool ProvidesTransponderExclusively(const cChannel *Channel) const;
          ///< Returns true if this is the only device that is able to provide
          ///< the given channel's transponder.
-  virtual bool ProvidesChannel(const cChannel *Channel, int Priority = -1, bool *NeedsDetachReceivers = NULL) const;
+  virtual bool ProvidesChannel(const cChannel *Channel, int Priority = IDLEPRIORITY, bool *NeedsDetachReceivers = NULL) const;
          ///< Returns true if this device can provide the given channel.
-         ///< In case the device has cReceivers attached to it or it is the primary
-         ///< device, Priority is used to decide whether the caller's request can
-         ///< be honored.
-         ///< The special Priority value -1 will tell the caller whether this device
+         ///< In case the device has cReceivers attached to it, Priority is used to
+         ///< decide whether the caller's request can be honored.
+         ///< The special Priority value IDLEPRIORITY will tell the caller whether this device
          ///< is principally able to provide the given Channel, regardless of any
          ///< attached cReceivers.
          ///< If NeedsDetachReceivers is given, the resulting value in it will tell the
@@ -272,12 +288,14 @@ public:
          ///< This is not one of the channels in the global cChannels list, but rather
          ///< a local copy. The result may be NULL if the device is not tuned to any
          ///< transponder.
-  virtual bool IsTunedToTransponder(const cChannel *Channel);
+  virtual bool IsTunedToTransponder(const cChannel *Channel) const;
          ///< Returns true if this device is currently tuned to the given Channel's
          ///< transponder.
-  virtual bool MaySwitchTransponder(void);
-         ///< Returns true if it is ok to switch the transponder on this device,
-         ///< without disturbing any other activities.
+  virtual bool MaySwitchTransponder(const cChannel *Channel) const;
+         ///< Returns true if it is ok to switch to the Channel's transponder on this
+         ///< device, without disturbing any other activities. If an occupied timeout
+         ///< has been set for this device, and that timeout has not yet expired,
+         ///< this function returns false,
   bool SwitchChannel(const cChannel *Channel, bool LiveView);
          ///< Switches the device to the given Channel, initiating transfer mode
          ///< if necessary.
@@ -300,6 +318,16 @@ public:
          ///< channel number while replaying.
   void ForceTransferMode(void);
          ///< Forces the device into transfermode for the current channel.
+  int Occupied(void) const;
+         ///< Returns the number of seconds this device is still occupied for.
+  void SetOccupied(int Seconds);
+         ///< Sets the occupied timeout for this device to the given number of
+         ///< Seconds, This can be used to tune a device to a particular transponder
+         ///< and make sure it will stay there for a certain amount of time, for
+         ///< instance to collect EPG data. This function shall only be called
+         ///< after the device has been successfully tuned to the requested transponder.
+         ///< Seconds will be silently limited to MAXOCCUPIEDTIMEOUT. Values less than
+         ///< 0 will be silently ignored.
   virtual bool HasLock(int TimeoutMs = 0);
          ///< Returns true if the device has a lock on the requested transponder.
          ///< Default is true, a specific device implementation may return false
@@ -339,6 +367,9 @@ protected:
          ///< Handle->used indicates how many receivers are using this PID.
          ///< Type indicates some special types of PIDs, which the device may
          ///< need to set in a specific way.
+public:
+  void DelLivePids(void);
+         ///< Deletes the live viewing PIDs.
 
 // Section filter facilities
 
@@ -362,6 +393,10 @@ public:
        ///< Opens a file handle for the given filter data.
        ///< A derived device that provides section data must
        ///< implement this function.
+  virtual int ReadFilter(int Handle, void *Buffer, size_t Length);
+       ///< Reads data from a handle for the given filter.
+       ///< A derived class need not implement this function, because this
+       ///< is done by the default implementation.
   virtual void CloseFilter(int Handle);
        ///< Closes a file handle that has previously been opened
        ///< by OpenFilter(). If this is as simple as calling close(Handle),
@@ -380,6 +415,12 @@ private:
 public:
   virtual bool HasCi(void);
          ///< Returns true if this device has a Common Interface.
+  virtual bool HasInternalCam(void) { return false; }
+         ///< Returns true if this device handles encrypted channels itself
+         ///< without VDR assistance. This can be e.g. if the device is a
+         ///< client that gets the stream from another VDR instance that has
+         ///< already decrypted the stream. In this case ProvidesChannel()
+         ///< shall check whether the channel can be decrypted.
   void SetCamSlot(cCamSlot *CamSlot);
          ///< Sets the given CamSlot to be used with this device.
   cCamSlot *CamSlot(void) const { return camSlot; }
@@ -421,14 +462,14 @@ public:
          ///< Returns the video system of the currently displayed material
          ///< (default is PAL).
   virtual void GetVideoSize(int &Width, int &Height, double &VideoAspect);
-         ///< Returns the With, Height and VideoAspect ratio of the currently
+         ///< Returns the Width, Height and VideoAspect ratio of the currently
          ///< displayed video material. Width and Height are given in pixel
          ///< (e.g. 720x576) and VideoAspect is e.g. 1.33333 for a 4:3 broadcast,
          ///< or 1.77778 for 16:9.
          ///< The default implementation returns 0 for Width and Height
          ///< and 1.0 for VideoAspect.
   virtual void GetOsdSize(int &Width, int &Height, double &PixelAspect);
-         ///< Returns the With, Height and PixelAspect ratio the OSD should use
+         ///< Returns the Width, Height and PixelAspect ratio the OSD should use
          ///< to best fit the resolution of the output device. If PixelAspect
          ///< is not 1.0, the OSD may take this as a hint to scale its
          ///< graphics in a way that, e.g., a circle will actually
@@ -457,7 +498,7 @@ protected:
        ///< Sets the current subtitle track to the given value.
 public:
   void ClrAvailableTracks(bool DescriptionsOnly = false, bool IdsOnly = false);
-       ///< Clears the list of currently availabe tracks. If DescriptionsOnly
+       ///< Clears the list of currently available tracks. If DescriptionsOnly
        ///< is true, only the track descriptions will be cleared. With IdsOnly
        ///< set to true only the ids will be cleared. IdsOnly is only taken
        ///< into account if DescriptionsOnly is false.
@@ -480,11 +521,11 @@ public:
        ///< is more than one audio track.
   int NumSubtitleTracks(void) const;
        ///< Returns the number of subtitle tracks that are currently available.
-  eTrackType GetCurrentAudioTrack(void) { return currentAudioTrack; }
+  eTrackType GetCurrentAudioTrack(void) const { return currentAudioTrack; }
   bool SetCurrentAudioTrack(eTrackType Type);
        ///< Sets the current audio track to the given Type.
        ///< \return Returns true if Type is a valid audio track, false otherwise.
-  eTrackType GetCurrentSubtitleTrack(void) { return currentSubtitleTrack; }
+  eTrackType GetCurrentSubtitleTrack(void) const { return currentSubtitleTrack; }
   bool SetCurrentSubtitleTrack(eTrackType Type, bool Manual = false);
        ///< Sets the current subtitle track to the given Type.
        ///< IF Manual is true, no automatic preferred subtitle language selection
@@ -651,10 +692,10 @@ public:
        ///< Returns true if the device itself or any of the file handles in
        ///< Poller is ready for further action.
        ///< If TimeoutMs is not zero, the device will wait up to the given number
-       ///< of milleseconds before returning in case it can't accept any data.
+       ///< of milliseconds before returning in case it can't accept any data.
   virtual bool Flush(int TimeoutMs = 0);
        ///< Returns true if the device's output buffers are empty, i. e. any
-       ///< data which was bufferd so far has been processed.
+       ///< data which was buffered so far has been processed.
        ///< If TimeoutMs is not zero, the device will wait up to the given
        ///< number of milliseconds before returning in case there is still
        ///< data in the buffers.
@@ -698,13 +739,12 @@ public:
 // Receiver facilities
 
 private:
-  cMutex mutexReceiver;
+  mutable cMutex mutexReceiver;
   cReceiver *receiver[MAXRECEIVERS];
 public:
   int Priority(void) const;
-      ///< Returns the priority of the current receiving session (0..MAXPRIORITY),
-      ///< or -1 if no receiver is currently active. The primary device will
-      ///< always return at least Setup.PrimaryLimit-1.
+      ///< Returns the priority of the current receiving session (-MAXPRIORITY..MAXPRIORITY),
+      ///< or IDLEPRIORITY if no receiver is currently active.
 protected:
   virtual bool OpenDvr(void);
       ///< Opens the DVR of this device and prepares it to deliver a Transport
@@ -719,15 +759,15 @@ protected:
       ///< false in case of a non recoverable error, otherwise it returns true,
       ///< even if Data is NULL.
 public:
-  bool Receiving(bool CheckAny = false) const;
-       ///< Returns true if we are currently receiving.
+  bool Receiving(bool Dummy = false) const;
+       ///< Returns true if we are currently receiving. The parameter has no meaning (for backwards compatibility only).
   bool AttachReceiver(cReceiver *Receiver);
        ///< Attaches the given receiver to this device.
   void Detach(cReceiver *Receiver);
        ///< Detaches the given receiver from this device.
   void DetachAll(int Pid);
        ///< Detaches all receivers from this device for this pid.
-  void DetachAllReceivers(void);
+  virtual void DetachAllReceivers(void);
        ///< Detaches all receivers from this device.
   };
 
