@@ -8,7 +8,7 @@
  * Robert Schneider <Robert.Schneider@web.de> and Rolf Hakenes <hakenes@hippomi.de>.
  * Adapted to 'libsi' for VDR 1.3.0 by Marcel Wiesweg <marcel.wiesweg@gmx.de>.
  *
- * $Id: eit.c 2.12 2010/05/14 14:08:35 kls Exp $
+ * $Id: eit.c 2.21 2012/08/25 11:13:00 kls Exp $
  */
 
 #include "eit.h"
@@ -32,28 +32,33 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
   if (!CheckCRCAndParse())
      return;
 
+  time_t Now = time(NULL);
+  if (Now < VALID_TIME)
+     return; // we need the current time for handling PDC descriptors
+
+  if (!Channels.Lock(false, 10))
+     return;
   tChannelID channelID(Source, getOriginalNetworkId(), getTransportStreamId(), getServiceId());
   cChannel *channel = Channels.GetByChannelID(channelID, true);
-  if (!channel)
-     return; // only collect data for known channels
+  if (!channel || EpgHandlers.IgnoreChannel(channel)) {
+     Channels.Unlock();
+     return;
+     }
 
+  bool handledExternally = EpgHandlers.HandledExternally(channel);
   cSchedule *pSchedule = (cSchedule *)Schedules->GetSchedule(channel, true);
 
   bool Empty = true;
   bool Modified = false;
-  bool HasExternalData = false;
   time_t SegmentStart = 0;
   time_t SegmentEnd = 0;
-  time_t Now = time(NULL);
   struct tm tm_r;
   struct tm t = *localtime_r(&Now, &tm_r); // this initializes the time zone in 't'
 
-  if (Now < VALID_TIME)
-     return; // we need the current time for handling PDC descriptors
-
   SI::EIT::Event SiEitEvent;
   for (SI::Loop::Iterator it; eventLoop.getNext(SiEitEvent, it); ) {
-      bool ExternalData = false;
+      if (EpgHandlers.HandleEitEvent(pSchedule, &SiEitEvent, Tid, getVersionNumber()))
+         continue; // an EPG handler has done all of the processing
       time_t StartTime = SiEitEvent.getStartTime();
       int Duration = SiEitEvent.getDuration();
       // Drop bogus events - but keep NVOD reference events, where all bits of the start time field are set to 1, resulting in a negative number.
@@ -66,105 +71,26 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
       cEvent *newEvent = NULL;
       cEvent *rEvent = NULL;
       cEvent *pEvent = (cEvent *)pSchedule->GetEvent(SiEitEvent.getEventId(), StartTime);
-      if (!pEvent) {
+      if (!pEvent || handledExternally) {
          if (OnlyRunningStatus)
             continue;
+         if (handledExternally && !EpgHandlers.IsUpdate(SiEitEvent.getEventId(), StartTime, Tid, getVersionNumber()))
+            continue;
          // If we don't have that event yet, we create a new one.
          // Otherwise we copy the information into the existing event anyway, because the data might have changed.
          pEvent = newEvent = new cEvent(SiEitEvent.getEventId());
-         if (!pEvent)
-            continue;
+         newEvent->SetStartTime(StartTime);
+         newEvent->SetDuration(Duration);
+         if (!handledExternally)
+            pSchedule->AddEvent(newEvent);
          }
       else {
          // We have found an existing event, either through its event ID or its start time.
          pEvent->SetSeen();
-         // If the existing event has a zero table ID it was defined externally and shall
-         // not be overwritten.
-         uchar TableID = pEvent->TableID();
-         if (TableID == 0x00) {
-            if (pEvent->Version() == getVersionNumber())
-          {
-          	if(Setup.MixEpgAction == 0)
-          		continue;
-          		
-          	//printf("in");
-          	//printf("%s", pEvent->GetTimeString());
-          	// to use the info of the original epg, update the extern one,
-          	// if it has less info
-          	SI::Descriptor *d;
-          	SI::ExtendedEventDescriptors *ExtendedEventDescriptors = NULL;
-          	//SI::ExtendedEventDescriptor *eed = NULL;
-          	SI::ShortEventDescriptor *ShortEventDescriptor = NULL;
-          	//SI::ShortEventDescriptor *sed = NULL;
-          	//SI::TimeShiftedEventDescriptor *tsed = NULL;
-          	//cLinkChannels *LinkChannels = NULL;
-          	for (SI::Loop::Iterator it2; (d = SiEitEvent.eventDescriptors.getNext(it2));)
-          	{
-          		if(d->getDescriptorTag() == SI::ShortEventDescriptorTag)
-          		{
-          			int LanguagePreferenceShort = -1;
-          			SI::ShortEventDescriptor *sed = (SI::ShortEventDescriptor *)d;
-          			if (I18nIsPreferredLanguage(Setup.EPGLanguages, sed->languageCode, LanguagePreferenceShort) || !ShortEventDescriptor)
-          			{
-          				delete ShortEventDescriptor;
-          				ShortEventDescriptor = sed;
-          				d = NULL; // so that it is not deleted
-          			}
-          		}
-          		else if(d->getDescriptorTag() == SI::ExtendedEventDescriptorTag)
-          		{
-          			int LanguagePreferenceExt = -1;
-          			bool UseExtendedEventDescriptor = false;
-          			SI::ExtendedEventDescriptor *eed = (SI::ExtendedEventDescriptor *)d;
-          			if (I18nIsPreferredLanguage(Setup.EPGLanguages, eed->languageCode, LanguagePreferenceExt) || !ExtendedEventDescriptors) 
-          			{
-          				delete ExtendedEventDescriptors;
-          				ExtendedEventDescriptors = new SI::ExtendedEventDescriptors;
-          				UseExtendedEventDescriptor = true;
-          			}
-          			if (UseExtendedEventDescriptor) 
-          			{
-          				ExtendedEventDescriptors->Add(eed);
-          				d = NULL; // so that it is not deleted
-          			}
-          			if (eed->getDescriptorNumber() == eed->getLastDescriptorNumber())
-          				UseExtendedEventDescriptor = false;
-          		}         		                 
-          		delete d;
-          	}
-          	if(pEvent)
-          	{
-          		
-             		if(ShortEventDescriptor)
- 	             	{
- 	             		char buffer[256];    		
- 	             		if(ShortEventDescriptor->text.getText(buffer, sizeof(buffer)) && pEvent->ShortText() && (strlen(ShortEventDescriptor->text.getText(buffer, sizeof(buffer))) > strlen(pEvent->ShortText())))
- 	             		{
- 	             			pEvent->SetShortText(ShortEventDescriptor->text.getText(buffer, sizeof(buffer)));
- 	             			pEvent->FixEpgBugs();
- 				}
- 			}
- 			if(ExtendedEventDescriptors)
- 			{
- 				char buffer[ExtendedEventDescriptors->getMaximumTextLength(": ") + 1];
- 				//pEvent->SetDescription(ExtendedEventDescriptors->getText(buffer, sizeof(buffer), ": "));
-             
- 				if(ExtendedEventDescriptors->getText(buffer, sizeof(buffer), ": ") && pEvent->Description() && (strlen(ExtendedEventDescriptors->getText(buffer, sizeof(buffer), ": ")) > strlen(pEvent->Description())))
- 				{
- 		  			pEvent->SetDescription(ExtendedEventDescriptors->getText(buffer, sizeof(buffer), ": "));
- 		  			pEvent->FixEpgBugs();
- 		  		}
- 		  	}
- 		}
- 	  	delete ExtendedEventDescriptors;
- 	  	delete ShortEventDescriptor;
-          	continue;
-          }
-            HasExternalData = ExternalData = true;
-            }
+         uchar TableID = max(pEvent->TableID(), uchar(0x4E)); // for backwards compatibility, table ids less than 0x4E are treated as if they were "present"
          // If the new event has a higher table ID, let's skip it.
          // The lower the table ID, the more "current" the information.
-         else if (Tid > TableID)
+         if (Tid > TableID)
             continue;
          // If the new event comes from the same table and has the same version number
          // as the existing one, let's skip it to avoid unnecessary work.
@@ -174,21 +100,20 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
          // to the Sat.1/Pro7 transponder, events will keep toggling because of the bogus version numbers.
          else if (Tid == TableID && pEvent->Version() == getVersionNumber())
             continue;
+         EpgHandlers.SetEventID(pEvent, SiEitEvent.getEventId()); // unfortunately some stations use different event ids for the same event in different tables :-(
+         EpgHandlers.SetStartTime(pEvent, StartTime);
+         EpgHandlers.SetDuration(pEvent, Duration);
          }
-      if (!ExternalData) {
-         pEvent->SetEventID(SiEitEvent.getEventId()); // unfortunately some stations use different event ids for the same event in different tables :-(
+      if (pEvent->TableID() > 0x4E) // for backwards compatibility, table ids less than 0x4E are never overwritten
          pEvent->SetTableID(Tid);
-         pEvent->SetStartTime(StartTime);
-         pEvent->SetDuration(Duration);
-         }
-      if (newEvent)
-         pSchedule->AddEvent(newEvent);
       if (Tid == 0x4E) { // we trust only the present/following info on the actual TS
-         if (Setup.DisableVPS == 0 && SiEitEvent.getRunningStatus() >= SI::RunningStatusNotRunning)
+         if (SiEitEvent.getRunningStatus() >= SI::RunningStatusNotRunning)
             pSchedule->SetRunningStatus(pEvent, SiEitEvent.getRunningStatus(), channel);
          }
-      if (OnlyRunningStatus)
+      if (OnlyRunningStatus) {
+         pEvent->SetVersion(0xFF); // we have already changed the table id above, so set the version to an invalid value to make sure the next full run will be executed
          continue; // do this before setting the version, so that the full update can be done later
+         }
       pEvent->SetVersion(getVersionNumber());
 
       int LanguagePreferenceShort = -1;
@@ -200,10 +125,6 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
       cLinkChannels *LinkChannels = NULL;
       cComponents *Components = NULL;
       for (SI::Loop::Iterator it2; (d = SiEitEvent.eventDescriptors.getNext(it2)); ) {
-          if (ExternalData && d->getDescriptorTag() != SI::ComponentDescriptorTag) {
-             delete d;
-             continue;
-             }
           switch (d->getDescriptorTag()) {
             case SI::ExtendedEventDescriptorTag: {
                  SI::ExtendedEventDescriptor *eed = (SI::ExtendedEventDescriptor *)d;
@@ -240,7 +161,7 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
                         NumContents++;
                         }
                      }
-                 pEvent->SetContents(Contents);
+                 EpgHandlers.SetContents(pEvent, Contents);
                  }
                  break;
             case SI::ParentalRatingDescriptorTag: {
@@ -259,7 +180,7 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
                           case 0x13:          ParentalRating = 16; break;
                           default:            ParentalRating = 0;
                           }
-                        pEvent->SetParentalRating(ParentalRating);
+                        EpgHandlers.SetParentalRating(pEvent, ParentalRating);
                         }
                      }
                  }
@@ -278,7 +199,7 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
                  else if (month == 0 && t.tm_mon == 11) // current month is jan, but event is in dec
                     t.tm_year--;
                  time_t vps = mktime(&t);
-                 pEvent->SetVps(vps);
+                 EpgHandlers.SetVps(pEvent, vps);
                  }
                  break;
             case SI::TimeShiftedEventDescriptorTag: {
@@ -289,9 +210,9 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
                  rEvent = (cEvent *)rSchedule->GetEvent(tsed->getReferenceEventId());
                  if (!rEvent)
                     break;
-                 pEvent->SetTitle(rEvent->Title());
-                 pEvent->SetShortText(rEvent->ShortText());
-                 pEvent->SetDescription(rEvent->Description());
+                 EpgHandlers.SetTitle(pEvent, rEvent->Title());
+                 EpgHandlers.SetShortText(pEvent, rEvent->ShortText());
+                 EpgHandlers.SetDescription(pEvent, rEvent->Description());
                  }
                  break;
             case SI::LinkageDescriptorTag: {
@@ -349,105 +270,32 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
       if (!rEvent) {
          if (ShortEventDescriptor) {
             char buffer[Utf8BufSize(256)];
-            pEvent->SetTitle(ShortEventDescriptor->name.getText(buffer, sizeof(buffer)));
-            pEvent->SetShortText(ShortEventDescriptor->text.getText(buffer, sizeof(buffer)));
+            EpgHandlers.SetTitle(pEvent, ShortEventDescriptor->name.getText(buffer, sizeof(buffer)));
+            EpgHandlers.SetShortText(pEvent, ShortEventDescriptor->text.getText(buffer, sizeof(buffer)));
             }
-         else if (!HasExternalData) {
-            pEvent->SetTitle(NULL);
-            pEvent->SetShortText(NULL);
+         else {
+            EpgHandlers.SetTitle(pEvent, NULL);
+            EpgHandlers.SetShortText(pEvent, NULL);
             }
          if (ExtendedEventDescriptors) {
             char buffer[Utf8BufSize(ExtendedEventDescriptors->getMaximumTextLength(": ")) + 1];
-            pEvent->SetDescription(ExtendedEventDescriptors->getText(buffer, sizeof(buffer), ": "));
+            EpgHandlers.SetDescription(pEvent, ExtendedEventDescriptors->getText(buffer, sizeof(buffer), ": "));
             }
-         else if (!HasExternalData)
-            pEvent->SetDescription(NULL);
+         else
+            EpgHandlers.SetDescription(pEvent, NULL);
          }
       delete ExtendedEventDescriptors;
       delete ShortEventDescriptor;
 
-      pEvent->SetComponents(Components);
+      EpgHandlers.SetComponents(pEvent, Components);
 
-      if (!HasExternalData)
-         pEvent->FixEpgBugs();
+      EpgHandlers.FixEpgBugs(pEvent);
       if (LinkChannels)
          channel->SetLinkChannels(LinkChannels);
       Modified = true;
-
-      //to avoid double epg-entrys from ext and int epg sources :EW
-      if (pEvent && pEvent->TableID() != 0x00)
-      {
-      	cEvent *pPreviousEvent = (cEvent *)pSchedule->GetPreviousEvent(pEvent);
-      	
-      	if (pPreviousEvent)
-       	{
-       		if(Setup.DoubleEpgAction == 0)
-       		{
-       			pPreviousEvent->SetStartTime(pEvent->StartTime());
-       			pPreviousEvent->SetDuration(pEvent->Duration());
-       			
-       			if(Setup.DisableVPS == 0)
-       			{
-       				if(channel)
-       					pPreviousEvent->SetRunningStatus(pEvent->RunningStatus(), channel);
-       				else
-       					pPreviousEvent->SetRunningStatus(pEvent->RunningStatus());
-       			}
-       			
-       			// to use the info of the original epg, update the extern one,
-       			// if it has less info
-       			char buffer_short_intern[256];
-       			char buffer_short_extern[256];
-       			int len_short_intern = 0;
-       			int len_short_extern = 0;
-       			
-       			if (pEvent->ShortText())
-       				len_short_intern = snprintf (buffer_short_intern, sizeof(buffer_short_intern), "%s", pEvent->ShortText());
-       				
-       			if (pPreviousEvent->ShortText())
-       				len_short_extern = snprintf (buffer_short_extern, sizeof(buffer_short_extern), "%s",pPreviousEvent->ShortText());
- 			
-       			if(len_short_intern > 0)
-       			{
-       				if(len_short_extern < 1)
-       					pPreviousEvent->SetShortText(buffer_short_intern);
-       				else if (len_short_intern > len_short_extern)
-       						pPreviousEvent->SetShortText(buffer_short_intern);
-       			}
-       			
-       			if(pEvent->Description())
-       			{
-       				char buffer_title_intern[4096];
-       				char buffer_title_extern[4096];
-       				int len_title_intern = 0;
-       				int len_title_extern = 0;
-       				
-       				if (pEvent->Description())
-       					len_title_intern = snprintf (buffer_title_intern, sizeof(buffer_title_intern), "%s", pEvent->Description());
-       					
-       				if (pPreviousEvent->Description())
-       					len_title_extern = snprintf (buffer_title_extern, sizeof(buffer_title_extern), "%s", pPreviousEvent->Description());
-       				
-       				if(len_title_intern > 0)
- 	      			{
- 	      				if(len_title_extern < 1)
- 	      					pPreviousEvent->SetDescription(buffer_title_intern);
- 	      				else if (len_title_intern > len_title_extern)
- 	      					pPreviousEvent->SetDescription(buffer_title_intern);
- 	      			}
- 	      		}
- 	      		
- 	      		if(pPreviousEvent->Vps() == 0 && pEvent->Vps() != 0)
- 	      			pPreviousEvent->SetVps(pEvent->Vps());
- 	      			
- 	      		pSchedule->DelEvent(pEvent);
- 	      		pPreviousEvent->FixEpgBugs();
-       		}
-       		else
-       			pSchedule->DelEvent(pPreviousEvent);
-       	}
-       }
-
+      EpgHandlers.HandleEvent(pEvent);
+      if (handledExternally)
+         delete pEvent;
       }
   if (Tid == 0x4E) {
      if (Empty && getSectionNumber() == 0)
@@ -455,14 +303,12 @@ cEIT::cEIT(cSchedules *Schedules, int Source, u_char Tid, const u_char *Data, bo
         pSchedule->ClrRunningStatus(channel);
      pSchedule->SetPresentSeen();
      }
-  if (OnlyRunningStatus)
-     return;
-  if (Modified) {
-     pSchedule->Sort();
-     if (!HasExternalData)
-        pSchedule->DropOutdated(SegmentStart, SegmentEnd, Tid, getVersionNumber());
+  if (Modified && !OnlyRunningStatus) {
+     EpgHandlers.SortSchedule(pSchedule);
+     EpgHandlers.DropOutdated(pSchedule, SegmentStart, SegmentEnd, Tid, getVersionNumber());
      Schedules->SetModified(pSchedule);
      }
+  Channels.Unlock();
 }
 
 // --- cTDT ------------------------------------------------------------------
