@@ -4,13 +4,14 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: osdbase.c 2.3 2010/12/12 13:41:28 kls Exp $
+ * $Id: osdbase.c 2.6 2012/04/23 09:41:22 kls Exp $
  */
 
 #include "osdbase.h"
 #include <string.h>
 #include "device.h"
 #include "i18n.h"
+#include "menuitems.h"
 #include "remote.h"
 #include "status.h"
 
@@ -71,7 +72,6 @@ void cOsdObject::Show(void)
 
 cSkinDisplayMenu *cOsdMenu::displayMenu = NULL;
 int cOsdMenu::displayMenuCount = 0;
-int cOsdMenu::displayMenuItems = 0;//XXX dynamic???
 
 cOsdMenu::cOsdMenu(const char *Title, int c0, int c1, int c2, int c3, int c4)
 {
@@ -79,13 +79,16 @@ cOsdMenu::cOsdMenu(const char *Title, int c0, int c1, int c2, int c3, int c4)
   digit = 0;
   key_nr = -1;
   hasHotkeys = false;
+  displayMenuItems = 0;
   title = NULL;
+  menuCategory = mcUnknown;
   SetTitle(Title);
   SetCols(c0, c1, c2, c3, c4);
   first = 0;
   current = marked = -1;
   subMenu = NULL;
   helpRed = helpGreen = helpYellow = helpBlue = NULL;
+  helpDisplayed = false;
   status = NULL;
   if (!displayMenuCount++)
      SetDisplayMenu();
@@ -102,6 +105,11 @@ cOsdMenu::~cOsdMenu()
      DELETENULL(displayMenu);
 }
 
+void cOsdMenu::SetMenuCategory(eMenuCategory MenuCategory)
+{
+  menuCategory = MenuCategory;
+}
+
 void cOsdMenu::SetDisplayMenu(void)
 {
   if (displayMenu) {
@@ -109,7 +117,6 @@ void cOsdMenu::SetDisplayMenu(void)
      delete displayMenu;
      }
   displayMenu = Skins.Current()->DisplayMenu();
-  displayMenuItems = displayMenu->MaxItems();
 }
 
 const char *cOsdMenu::hk(const char *s)
@@ -155,6 +162,15 @@ void cOsdMenu::SetTitle(const char *Title)
   title = strdup(Title);
 }
 
+void cOsdMenu::DisplayHelp(bool Force)
+{
+  if (!helpDisplayed || Force) {
+     displayMenu->SetButtons(helpRed, helpGreen, helpYellow, helpBlue);
+     cStatus::MsgOsdHelpKeys(helpRed, helpGreen, helpYellow, helpBlue);
+     helpDisplayed = true;
+     }
+}
+
 void cOsdMenu::SetHelp(const char *Red, const char *Green, const char *Yellow, const char *Blue)
 {
   // strings are NOT copied - must be constants!!!
@@ -162,8 +178,7 @@ void cOsdMenu::SetHelp(const char *Red, const char *Green, const char *Yellow, c
   helpGreen  = Green;
   helpYellow = Yellow;
   helpBlue   = Blue;
-  displayMenu->SetButtons(helpRed, helpGreen, helpYellow, helpBlue);
-  cStatus::MsgOsdHelpKeys(helpRed, helpGreen, helpYellow, helpBlue);
+  DisplayHelp(true);
 }
 
 void cOsdMenu::Del(int Index)
@@ -203,11 +218,13 @@ void cOsdMenu::Display(void)
   displayMenu->SetMessage(mtStatus, NULL);
   displayMenu->Clear();
   cStatus::MsgOsdClear();
+  if (menuCategory != displayMenu->MenuCategory())
+     displayMenu->SetMenuCategory(menuCategory);
+  displayMenuItems = displayMenu->MaxItems();
   displayMenu->SetTabs(cols[0], cols[1], cols[2], cols[3], cols[4]);//XXX
   displayMenu->SetTitle(title);
   cStatus::MsgOsdTitle(title);
-  displayMenu->SetButtons(helpRed, helpGreen, helpYellow, helpBlue);
-  cStatus::MsgOsdHelpKeys(helpRed, helpGreen, helpYellow, helpBlue);
+  DisplayHelp(true);
   int count = Count();
   if (count > 0) {
      int ni = 0;
@@ -264,6 +281,12 @@ void cOsdMenu::DisplayCurrent(bool Current)
         cStatus::MsgOsdCurrentItem(item->Text());
      if (!Current)
         item->SetFresh(true); // leaving the current item resets 'fresh'
+     if (cMenuEditItem *MenuEditItem = dynamic_cast<cMenuEditItem *>(item)) {
+        if (!MenuEditItem->DisplayHelp())
+           DisplayHelp();
+        else
+           helpDisplayed = false;
+        }
      }
 }
 
@@ -465,9 +488,8 @@ eOSState cOsdMenu::HotKey(eKeys Key)
      else
         return osContinue;
      }
-  else {
+  else
      lastActivity.Set(MENUKEY_TIMEOUT);
-     }
   for (cOsdItem *item = Last(); item; item = Prev(item)) {
       const char *s = item->Text();
       i = 0;
@@ -503,9 +525,8 @@ eOSState cOsdMenu::HotKey(eKeys Key)
             }
          }
       }
-  if ((!match) && (Key != kNone)) {
+  if ((!match) && (Key != kNone))
      key_nr = -1;
-     }
   return osContinue;
 }
 
