@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: recording.c 2.38 2011/09/04 09:32:25 kls Exp $
+ * $Id: recording.c 2.64 2012/09/30 13:05:14 kls Exp $
  */
 
 #include "recording.h"
@@ -53,6 +53,8 @@
 #define INFOFILESUFFIX    "/info"
 #define MARKSFILESUFFIX   "/marks"
 
+#define SORTMODEFILE      ".sort"
+
 #define MINDISKSPACE 1024 // MB
 
 #define REMOVECHECKDELTA   60 // seconds between checks for removing deleted files
@@ -67,7 +69,6 @@
 #define MAX_LINK_LEVEL  6
 
 bool VfatFileSystem = false;
-bool DirOrderState = false;
 int InstanceId = 0;
 
 cRecordings DeletedRecordings(true);
@@ -96,6 +97,8 @@ void cRemoveDeletedRecordingsThread::Action(void)
      bool deleted = false;
      cThreadLock DeletedRecordingsLock(&DeletedRecordings);
      for (cRecording *r = DeletedRecordings.First(); r; ) {
+         if (cIoThrottle::Engaged())
+           return;
          if (r->Deleted() && time(NULL) - r->Deleted() > DELETEDLIFETIME) {
             cRecording *next = DeletedRecordings.Next(r);
             r->Remove();
@@ -106,8 +109,10 @@ void cRemoveDeletedRecordingsThread::Action(void)
             }
          r = DeletedRecordings.Next(r);
          }
-     if (deleted)
-        RemoveEmptyVideoDirectories();
+     if (deleted) {
+        const char *IgnoreFiles[] = { SORTMODEFILE, NULL };
+        RemoveEmptyVideoDirectories(IgnoreFiles);
+        }
      }
 }
 
@@ -154,7 +159,7 @@ void AssertFreeDiskSpace(int Priority, bool Force)
            cRecording *r = DeletedRecordings.First();
            cRecording *r0 = NULL;
            while (r) {
-                 if (IsOnVideoDirectoryFileSystem(r->FileName())) { // only remove recordings that will actually increase the free video disk space
+                 if (r->IsOnVideoDirectoryFileSystem()) { // only remove recordings that will actually increase the free video disk space
                     if (!r0 || r->Start() < r0->Start())
                        r0 = r;
                     }
@@ -181,7 +186,7 @@ void AssertFreeDiskSpace(int Priority, bool Force)
            cRecording *r = Recordings.First();
            cRecording *r0 = NULL;
            while (r) {
-                 if (IsOnVideoDirectoryFileSystem(r->FileName())) { // only delete recordings that will actually increase the free video disk space
+                 if (r->IsOnVideoDirectoryFileSystem()) { // only delete recordings that will actually increase the free video disk space
                     if (!r->IsEdited() && r->Lifetime() < MAXLIFETIME) { // edited recordings and recordings with MAXLIFETIME live forever
                        if ((r->Lifetime() == 0 && Priority > r->Priority()) || // the recording has no guaranteed lifetime and the new recording has higher priority
                            (r->Lifetime() > 0 && (time(NULL) - r->Start()) / SECSINDAY >= r->Lifetime())) { // the recording's guaranteed lifetime has expired
@@ -611,16 +616,14 @@ cRecording::cRecording(cTimer *Timer, const cEvent *Event)
 {
   resume = RESUME_NOT_INITIALIZED;
   titleBuffer = NULL;
-  for (int i = 0; i < MAXSORTMODES; i++) {
-     sortBuffer[i] = NULL;
-     lastDirsFirst[i] = -1;
-  }
+  sortBufferName = sortBufferTime = NULL;
   fileName = NULL;
   name = NULL;
   fileSizeMB = -1; // unknown
   channel = Timer->Channel()->Number();
   instanceId = InstanceId;
   isPesRecording = false;
+  isOnVideoDirectoryFileSystem = -1; // unknown
   framesPerSecond = DEFAULTFRAMESPERSECOND;
   numFrames = -1;
   deleted = 0;
@@ -681,14 +684,12 @@ cRecording::cRecording(const char *FileName)
   priority = MAXPRIORITY; // assume maximum in case there is no info file
   lifetime = MAXLIFETIME;
   isPesRecording = false;
+  isOnVideoDirectoryFileSystem = -1; // unknown
   framesPerSecond = DEFAULTFRAMESPERSECOND;
   numFrames = -1;
   deleted = 0;
   titleBuffer = NULL;
-  for (int i = 0; i < MAXSORTMODES; i++) {
-     sortBuffer[i] = NULL;
-     lastDirsFirst[i] = -1;
-  }
+  sortBufferName = sortBufferTime = NULL;
   FileName = fileName = strdup(FileName);
   if (*(fileName + strlen(fileName) - 1) == '/')
      *(fileName + strlen(fileName) - 1) = 0;
@@ -730,7 +731,9 @@ cRecording::cRecording(const char *FileName)
            }
         fclose(f);
         }
-     else if (errno != ENOENT)
+     else if (errno == ENOENT)
+        info->ownEvent->SetTitle(name);
+     else
         LOG_ERROR_STR(*InfoFileName);
 #ifdef SUMMARYFALLBACK
      // fall back to the old 'summary.vdr' if there was no 'info.vdr':
@@ -798,9 +801,8 @@ cRecording::cRecording(const char *FileName)
 cRecording::~cRecording()
 {
   free(titleBuffer);
-  for (int i = 0; i < MAXSORTMODES; i++) {
-     free(sortBuffer[i]);
-  }
+  free(sortBufferName);
+  free(sortBufferTime);
   free(fileName);
   free(name);
   delete info;
@@ -822,31 +824,30 @@ char *cRecording::StripEpisodeName(char *s)
         t++;
         }
   if (s1 && s2) {
-    if (Setup.RecordingsSortDirsFirst)
-      *s1 = 'b';
-
-    if ((Setup.RecordingsSortMode <= 1 && s1 != s && !strchr(".-$", *(s1 - 1))) ||
-        (Setup.RecordingsSortMode == 1 && s1 == s) ||
-        (Setup.RecordingsSortMode == 3))
-     memmove(s1 + 1, s2, t - s2 + 1);
-  }
+     // To have folders sorted before plain recordings, the '/' s1 points to
+     // is replaced by the character 'b'. All other slashes will be replaced
+     // by 'a' in SortName() (see below), which will result in the desired
+     // sequence:
+     *s1 = 'b';
+     s1++;
+     memmove(s1, s2, t - s2 + 1);
+     }
   return s;
 }
 
 char *cRecording::SortName(void) const
 {
-  if (!sortBuffer[Setup.RecordingsSortMode] ||
-        lastDirsFirst[Setup.RecordingsSortMode] != Setup.RecordingsSortDirsFirst) {
-     free(sortBuffer[Setup.RecordingsSortMode]);
-     lastDirsFirst[Setup.RecordingsSortMode] = Setup.RecordingsSortDirsFirst;
-     char *s = StripEpisodeName(strdup(FileName() + strlen(VideoDirectory)));
+  char **sb = (RecordingsSortMode == rsmName) ? &sortBufferName : &sortBufferTime;
+  if (!*sb) {
+     char *s = (RecordingsSortMode == rsmName) ? strdup(FileName() + strlen(VideoDirectory))
+                                              : StripEpisodeName(strdup(FileName() + strlen(VideoDirectory)));
      strreplace(s, '/', 'a'); // some locales ignore '/' when sorting
      int l = strxfrm(NULL, s, 0) + 1;
-     sortBuffer[Setup.RecordingsSortMode] = MALLOC(char, l);
-     strxfrm(sortBuffer[Setup.RecordingsSortMode], s, l);
+     *sb = MALLOC(char, l);
+     strxfrm(*sb, s, l);
      free(s);
      }
-  return sortBuffer[Setup.RecordingsSortMode];
+  return *sb;
 }
 
 int cRecording::GetResume(void) const
@@ -861,7 +862,7 @@ int cRecording::GetResume(void) const
 int cRecording::Compare(const cListObject &ListObject) const
 {
   cRecording *r = (cRecording *)&ListObject;
-  return Recordings.GetSortOrder() * strcasecmp(SortName(), r->SortName());
+  return strcasecmp(SortName(), r->SortName());
 }
 
 const char *cRecording::FileName(void) const
@@ -879,7 +880,7 @@ const char *cRecording::FileName(void) const
   return fileName;
 }
 
-const char *cRecording::Title(char Delimiter, bool NewIndicator, int Level, bool Original) const
+const char *cRecording::Title(char Delimiter, bool NewIndicator, int Level) const
 {
   char New = NewIndicator && IsNew() ? '*' : ' ';
   free(titleBuffer);
@@ -892,41 +893,26 @@ const char *cRecording::Title(char Delimiter, bool NewIndicator, int Level, bool
         s++;
      else
         s = name;
-     if (Original) {
-     titleBuffer = strdup(cString::sprintf("%02d.%02d.%02d%c%02d:%02d%c%d:%02d%c%c%s",
+     cString Length("");
+     if (NewIndicator) {
+        int Minutes = max(0, (LengthInSeconds() + 30) / 60);
+        Length = cString::sprintf("%c%d:%02d",
+                   Delimiter,
+                   Minutes / 60,
+                   Minutes % 60
+                   );
+        }
+     titleBuffer = strdup(cString::sprintf("%02d.%02d.%02d%c%02d:%02d%s%c%c%s",
                             t->tm_mday,
                             t->tm_mon + 1,
                             t->tm_year % 100,
                             Delimiter,
                             t->tm_hour,
                             t->tm_min,
-                            Delimiter,
-                            (LengthInSeconds() >= 0) ? LengthInSeconds() / 3600 : 0,
-                            (LengthInSeconds() >= 0) ? LengthInSeconds() / 60 % 60 : 0,
+                            *Length,
                             New,
                             Delimiter,
                             s));
-        }
-     else {
-        cString RecLength("---");
-        if (Setup.ShowRecLength && FileName()) {
-           int length = cIndexFile::GetLength(FileName(), IsPesRecording());
-           if (length >= 0)
-              RecLength = cString::sprintf("%d'", length / SecondsToFrames(60, framesPerSecond));
-           }
-        cString RecDate = cString::sprintf("%02d.%02d.%02d", t->tm_mday, t->tm_mon + 1, t->tm_year % 100);
-        cString RecTime = cString::sprintf("%02d:%02d", t->tm_hour, t->tm_min);
-        cString RecDelimiter = cString::sprintf("%c", Delimiter);
-        titleBuffer = strdup(cString::sprintf("%s%s%s%c%s%s%s%s",
-                               (Setup.ShowRecDate ? *RecDate        : ""),
-                               (Setup.ShowRecDate && Setup.ShowRecTime ? *RecDelimiter : ""),
-                               (Setup.ShowRecTime ? *RecTime        : ""),
-                               New,
-                               (Setup.ShowRecTime || Setup.ShowRecDate ? *RecDelimiter : ""),
-                               (Setup.ShowRecLength ? *RecLength    : ""),
-                               (Setup.ShowRecLength ? *RecDelimiter : ""),
-                               s));
-        }
      // let's not display a trailing FOLDERDELIMCHAR:
      if (!NewIndicator)
         stripspace(titleBuffer);
@@ -966,6 +952,16 @@ const char *cRecording::PrefixFileName(char Prefix)
   return NULL;
 }
 
+const char *cRecording::UpdateFileName(const char *FileName)
+{
+  if (FileName && *FileName) {
+     free(fileName);
+     fileName = strdup(FileName);
+     return fileName;
+     }
+  return NULL;
+}
+
 int cRecording::HierarchyLevels(void) const
 {
   const char *s = name;
@@ -984,6 +980,13 @@ bool cRecording::IsEdited(void) const
   return *s == '%';
 }
 
+bool cRecording::IsOnVideoDirectoryFileSystem(void) const
+{
+  if (isOnVideoDirectoryFileSystem < 0)
+     isOnVideoDirectoryFileSystem = ::IsOnVideoDirectoryFileSystem(FileName());
+  return isOnVideoDirectoryFileSystem;
+}
+
 void cRecording::ReadInfo(void)
 {
   info->Read();
@@ -1025,8 +1028,10 @@ bool cRecording::Delete(void)
         RemoveVideoFile(NewName);
         }
      isyslog("deleting recording '%s'", FileName());
-     if (access(FileName(), F_OK) == 0)
+     if (access(FileName(), F_OK) == 0) {
         result = RenameVideoFile(FileName(), NewName);
+        cRecordingUserCommand::InvokeCommand(RUC_DELETERECORDING, NewName);
+        }
      else {
         isyslog("recording '%s' vanished", FileName());
         result = true; // well, we were going to delete it, anyway
@@ -1082,7 +1087,7 @@ int cRecording::NumFrames(void) const
 {
   if (numFrames < 0) {
      int nf = cIndexFile::GetLength(FileName(), IsPesRecording());
-     if (time(NULL) - LastModifiedTime(FileName()) < MININDEXAGE)
+     if (time(NULL) - LastModifiedTime(cIndexFile::IndexFileName(FileName(), IsPesRecording())) < MININDEXAGE)
         return nf; // check again later for ongoing recordings
      numFrames = nf;
      }
@@ -1093,44 +1098,19 @@ int cRecording::LengthInSeconds(void) const
 {
   int nf = NumFrames();
   if (nf >= 0)
-     return int((nf / FramesPerSecond() + 30) / 60) * 60;
+     return int(nf / FramesPerSecond());
   return -1;
 }
 
-bool cRecording::Rename(const char *newName)
+int cRecording::FileSizeMB(void) const
 {
-  bool result = false;
-  struct tm tm_r;
-  struct tm *t = localtime_r(&start, &tm_r);
-  char *localNewName = ExchangeChars(strdup(newName), true);
-  const char *fmt = isPesRecording ? NAMEFORMATPES : NAMEFORMATTS;
-  int ch = isPesRecording ? priority : channel;
-  int ri = isPesRecording ? lifetime : instanceId;
-  char *newFileName = strdup(cString::sprintf(fmt, VideoDirectory, localNewName, t->tm_year + 1900, t->tm_mon + 1, t->tm_mday, t->tm_hour, t->tm_min, ch, ri));
-  free(localNewName);
-  if (strcmp(FileName(), newFileName)) {
-     if (access(newFileName, F_OK) == 0) {
-        esyslog("recording %s already exists", newFileName);
-        }
-     else {
-        isyslog("renaming recording %s to %s", FileName(), newFileName);
-        result = MoveVideoFile(FileName(), newFileName);
-        if (result) {
-           free(fileName);
-           fileName = strdup(newFileName);
-           free(name);
-           name = strdup(newName);
-           for (int i = 0; i < MAXSORTMODES; i++) {
-              free(sortBuffer[i]);
-              sortBuffer[i] = NULL;
-           }
-           free(titleBuffer);
-           titleBuffer = NULL;
-           }
-        }
+  if (fileSizeMB < 0) {
+     int fs = DirSizeMB(FileName());
+     if (time(NULL) - LastModifiedTime(cIndexFile::IndexFileName(FileName(), IsPesRecording())) < MININDEXAGE)
+        return fs; // check again later for ongoing recordings
+     fileSizeMB = fs;
      }
-  free(newFileName);
-  return result;
+  return fileSizeMB;
 }
 
 // --- cRecordings -----------------------------------------------------------
@@ -1145,7 +1125,6 @@ cRecordings::cRecordings(bool Deleted)
   deleted = Deleted;
   lastUpdate = 0;
   state = 0;
-  SortOrder = 1;
 }
 
 cRecordings::~cRecordings()
@@ -1180,49 +1159,38 @@ void cRecordings::ScanVideoDir(const char *DirName, bool Foreground, int LinkLev
   cReadDir d(DirName);
   struct dirent *e;
   while ((Foreground || Running()) && (e = d.Next()) != NULL) {
-        if (strcmp(e->d_name, ".") && strcmp(e->d_name, "..")) {
-           char *buffer = strdup(AddDirectory(DirName, e->d_name));
-           struct stat st;
-           if (stat(buffer, &st) == 0) {
-              int Link = 0;
-              if (S_ISLNK(st.st_mode)) {
-                 if (LinkLevel > MAX_LINK_LEVEL) {
-                    isyslog("max link level exceeded - not scanning %s", buffer);
-                    continue;
-                    }
-                 Link = 1;
-                 char *old = buffer;
-                 buffer = ReadLink(old);
-                 free(old);
-                 if (!buffer)
-                    continue;
-                 if (stat(buffer, &st) != 0) {
-                    free(buffer);
-                    continue;
-                    }
+        cString buffer = AddDirectory(DirName, e->d_name);
+        struct stat st;
+        if (lstat(buffer, &st) == 0) {
+           int Link = 0;
+           if (S_ISLNK(st.st_mode)) {
+              if (LinkLevel > MAX_LINK_LEVEL) {
+                 isyslog("max link level exceeded - not scanning %s", *buffer);
+                 continue;
                  }
-              if (S_ISDIR(st.st_mode)) {
-                 if (endswith(buffer, deleted ? DELEXT : RECEXT)) {
-                    cRecording *r = new cRecording(buffer);
-                    if (r->Name()) {
-                       r->NumFrames(); // initializes the numFrames member
-                       Lock();
-                       Add(r);
-                       ChangeState();
-                       Unlock();
-                       if (deleted) {
-                          r->fileSizeMB = DirSizeMB(buffer);
-                          r->deleted = time(NULL);
-                          }
-                       }
-                    else
-                       delete r;
+              Link = 1;
+              if (stat(buffer, &st) != 0)
+                 continue;
+              }
+           if (S_ISDIR(st.st_mode)) {
+              if (endswith(buffer, deleted ? DELEXT : RECEXT)) {
+                 cRecording *r = new cRecording(buffer);
+                 if (r->Name()) {
+                    r->NumFrames(); // initializes the numFrames member
+                    r->FileSizeMB(); // initializes the fileSizeMB member
+                    if (deleted)
+                       r->deleted = time(NULL);
+                    Lock();
+                    Add(r);
+                    ChangeState();
+                    Unlock();
                     }
                  else
-                    ScanVideoDir(buffer, Foreground, LinkLevel + Link);
+                    delete r;
                  }
+              else
+                 ScanVideoDir(buffer, Foreground, LinkLevel + Link);
               }
-           free(buffer);
            }
         }
 }
@@ -1240,7 +1208,7 @@ void cRecordings::TouchUpdate(void)
   bool needsUpdate = NeedsUpdate();
   TouchFile(UpdateFileName());
   if (!needsUpdate)
-     lastUpdate = time(NULL); // make sure we don't tigger ourselves
+     lastUpdate = time(NULL); // make sure we don't trigger ourselves
 }
 
 bool cRecordings::NeedsUpdate(void)
@@ -1286,7 +1254,7 @@ void cRecordings::AddByName(const char *FileName, bool TriggerUpdate)
      }
 }
 
-void cRecordings::DelByName(const char *FileName)
+void cRecordings::DelByName(const char *FileName, bool RemoveRecording)
 {
   LOCK_THREAD;
   cRecording *recording = GetByName(FileName);
@@ -1294,14 +1262,15 @@ void cRecordings::DelByName(const char *FileName)
      cThreadLock DeletedRecordingsLock(&DeletedRecordings);
      Del(recording, false);
      char *ext = strrchr(recording->fileName, '.');
-     if (ext) {
+     if (ext && RemoveRecording) {
         strncpy(ext, DELEXT, strlen(ext));
-        recording->fileSizeMB = DirSizeMB(recording->FileName());
-        recording->deleted = time(NULL);
-        DeletedRecordings.Add(recording);
+        if (access(recording->FileName(), F_OK) == 0) {
+           recording->deleted = time(NULL);
+           DeletedRecordings.Add(recording);
+           recording = NULL; // to prevent it from being deleted below
+           }
         }
-     else
-        delete recording;
+     delete recording;
      ChangeState();
      TouchUpdate();
      }
@@ -1320,12 +1289,33 @@ int cRecordings::TotalFileSizeMB(void)
   int size = 0;
   LOCK_THREAD;
   for (cRecording *recording = First(); recording; recording = Next(recording)) {
-      if (recording->fileSizeMB > 0 && IsOnVideoDirectoryFileSystem(recording->FileName()))
-         size += recording->fileSizeMB;
+      int FileSizeMB = recording->FileSizeMB();
+      if (FileSizeMB > 0 && recording->IsOnVideoDirectoryFileSystem())
+         size += FileSizeMB;
       }
   return size;
 }
 
+double cRecordings::MBperMinute(void)
+{
+  int size = 0;
+  int length = 0;
+  LOCK_THREAD;
+  for (cRecording *recording = First(); recording; recording = Next(recording)) {
+      if (recording->IsOnVideoDirectoryFileSystem()) {
+         int FileSizeMB = recording->FileSizeMB();
+         if (FileSizeMB > 0) {
+            int LengthInSeconds = recording->LengthInSeconds();
+            if (LengthInSeconds > 0) {
+               size += FileSizeMB;
+               length += LengthInSeconds;
+               }
+            }
+         }
+      }
+  return (size && length) ? double(size) * 60 / length : -1;
+}
+
 void cRecordings::ResetResume(const char *ResumeFileName)
 {
   LOCK_THREAD;
@@ -1471,13 +1461,17 @@ cMark *cMarks::GetNext(int Position)
 
 const char *cRecordingUserCommand::command = NULL;
 
-void cRecordingUserCommand::InvokeCommand(const char *State, const char *RecordingFileName)
+void cRecordingUserCommand::InvokeCommand(const char *State, const char *RecordingFileName, const char *SourceFileName)
 {
   if (command) {
-     cString cmd = cString::sprintf("%s %s \"%s\"", command, State, *strescape(RecordingFileName, "\\\"$"));
-     isyslog("executing '%s'", *cmd);
-     SystemExec(cmd);
-     }
+    cString cmd;
+    if (SourceFileName)
+       cmd = cString::sprintf("%s %s \"%s\" \"%s\"", command, State, *strescape(RecordingFileName, "\\\"$"), *strescape(SourceFileName, "\\\"$"));
+    else
+       cmd = cString::sprintf("%s %s \"%s\"", command, State, *strescape(RecordingFileName, "\\\"$"));
+    isyslog("executing '%s'", *cmd);
+    SystemExec(cmd);
+  }
 }
 
 // --- cIndexFileGenerator ---------------------------------------------------
@@ -1509,6 +1503,7 @@ cIndexFileGenerator::~cIndexFileGenerator()
 void cIndexFileGenerator::Action(void)
 {
   bool IndexFileComplete = false;
+  bool IndexFileWritten = false;
   bool Rewind = false;
   cFileName FileName(recordingName, false);
   cUnbufferedFile *ReplayFile = FileName.Open();
@@ -1540,6 +1535,7 @@ void cIndexFileGenerator::Action(void)
                  if (FrameDetector.NewFrame()) {
                     IndexFile.Write(FrameDetector.IndependentFrame(), FileName.Number(), FrameOffset >= 0 ? FrameOffset : FileSize);
                     FrameOffset = -1;
+                    IndexFileWritten = true;
                     }
                  FileSize += Processed;
                  Buffer.Del(Processed);
@@ -1594,20 +1590,22 @@ void cIndexFileGenerator::Action(void)
            break;
            }
         }
+  if (IndexFileComplete) {
+     if (IndexFileWritten) {
+        Skins.QueueMessage(mtInfo, tr("Index file regeneration complete"));
+        return;
+        }
+     else
+        Skins.QueueMessage(mtError, tr("Index file regeneration failed!"));
+     }
   // Delete the index file if the recording has not been processed entirely:
-  if (IndexFileComplete)
-     Skins.QueueMessage(mtInfo, tr("Index file regeneration complete"));
-  else
-     IndexFile.Delete();
+  IndexFile.Delete();
 }
 
 // --- cIndexFile ------------------------------------------------------------
 
 #define INDEXFILESUFFIX     "/index"
 
-// The number of frames to stay off the end in case of time shift:
-#define INDEXSAFETYLIMIT 150 // frames
-
 // The maximum time to wait before giving up while catching up on an index file:
 #define MAXINDEXCATCHUP   8 // seconds
 
@@ -1634,8 +1632,12 @@ struct tIndexTs {
 
 #define MAXWAITFORINDEXFILE     10 // max. time to wait for the regenerated index file (seconds)
 #define INDEXFILECHECKINTERVAL 500 // ms between checks for existence of the regenerated index file
+#define INDEXFILETESTINTERVAL   10 // ms between tests for the size of the index file in case of pausing live video
 
-cIndexFile::cIndexFile(const char *FileName, bool Record, bool IsPesRecording)
+cMutex cIndexFile::indexListMutex;
+cVector<const cIndexFile *> cIndexFile::indexList;
+
+cIndexFile::cIndexFile(const char *FileName, bool Record, bool IsPesRecording, bool PauseLive)
 :resumeFile(FileName, IsPesRecording)
 {
   f = -1;
@@ -1646,6 +1648,12 @@ cIndexFile::cIndexFile(const char *FileName, bool Record, bool IsPesRecording)
   indexFileGenerator = NULL;
   if (FileName) {
      fileName = IndexFileName(FileName, isPesRecording);
+     if (!Record && PauseLive) {
+        // Wait until the index file contains at least two frames:
+        time_t tmax = time(NULL) + MAXWAITFORINDEXFILE;
+        while (time(NULL) < tmax && FileSize(fileName) < off_t(2 * sizeof(tIndexTs)))
+              cCondWait::SleepMs(INDEXFILETESTINTERVAL);
+        }
      int delta = 0;
      if (!Record && access(fileName, R_OK) != 0) {
         // Index file doesn't exist, so try to regenerate it:
@@ -1709,10 +1717,13 @@ cIndexFile::cIndexFile(const char *FileName, bool Record, bool IsPesRecording)
            LOG_ERROR_STR(*fileName);
         }
      }
+  if (Record)
+     AddToIndexList(this);
 }
 
 cIndexFile::~cIndexFile()
 {
+  RemoveFromIndexList(this);
   if (f >= 0)
      close(f);
   free(index);
@@ -1757,8 +1768,7 @@ bool cIndexFile::CatchUp(int Index)
      for (int i = 0; i <= MAXINDEXCATCHUP && (Index < 0 || Index >= last); i++) {
          struct stat buf;
          if (fstat(f, &buf) == 0) {
-            if (time(NULL) - buf.st_mtime > MININDEXAGE) {
-               // apparently the index file is not being written any more
+            if (!IsInIndexList(this)) {
                close(f);
                f = -1;
                break;
@@ -1800,7 +1810,7 @@ bool cIndexFile::CatchUp(int Index)
             }
          else
             LOG_ERROR_STR(*fileName);
-         if (Index < last - (i ? 2 * INDEXSAFETYLIMIT : 0) || Index > 10 * INDEXSAFETYLIMIT) // keep off the end in case of "Pause live video"
+         if (Index < last)
             break;
          cCondWait::SleepMs(1000);
          }
@@ -1847,13 +1857,13 @@ bool cIndexFile::Get(int Index, uint16_t *FileNumber, off_t *FileOffset, bool *I
   return false;
 }
 
-int cIndexFile::GetNextIFrame(int Index, bool Forward, uint16_t *FileNumber, off_t *FileOffset, int *Length, bool StayOffEnd)
+int cIndexFile::GetNextIFrame(int Index, bool Forward, uint16_t *FileNumber, off_t *FileOffset, int *Length)
 {
   if (CatchUp()) {
      int d = Forward ? 1 : -1;
      for (;;) {
          Index += d;
-         if (Index >= 0 && Index < last - ((Forward && StayOffEnd) ? INDEXSAFETYLIMIT : 0)) {
+         if (Index >= 0 && Index < last) {
             if (index[Index].independent) {
                uint16_t fn;
                if (!FileNumber)
@@ -1924,6 +1934,39 @@ int cIndexFile::GetLength(const char *FileName, bool IsPesRecording)
   return -1;
 }
 
+void cIndexFile::AddToIndexList(const cIndexFile *IndexFile)
+{
+  cMutexLock MutexLock(&indexListMutex);
+  for (int i = 0; i < indexList.Size(); i++) {
+      if (!indexList[i]) {
+         indexList[i] = IndexFile;
+         return;
+         }
+      }
+  indexList.Append(IndexFile);
+}
+
+void cIndexFile::RemoveFromIndexList(const cIndexFile *IndexFile)
+{
+  cMutexLock MutexLock(&indexListMutex);
+  for (int i = 0; i < indexList.Size(); i++) {
+      if (indexList[i] == IndexFile) {
+         indexList[i] = NULL;
+         return;
+         }
+      }
+}
+
+bool cIndexFile::IsInIndexList(const cIndexFile *IndexFile)
+{
+  cMutexLock MutexLock(&indexListMutex);
+  for (int i = 0; i < indexList.Size(); i++) {
+      if (indexList[i] && !strcmp(indexList[i]->fileName, IndexFile->fileName))
+         return true;
+      }
+  return false;
+}
+
 bool GenerateIndex(const char *FileName) 
 {
   if (DirectoryOk(FileName)) {
@@ -2132,15 +2175,18 @@ cUnbufferedFile *cFileName::NextFile(void)
 
 cString IndexToHMSF(int Index, bool WithFrame, double FramesPerSecond)
 {
-  char buffer[16];
+  const char *Sign = "";
+  if (Index < 0) {
+     Index = -Index;
+     Sign = "-";
+     }
   double Seconds;
   int f = int(modf((Index + 0.5) / FramesPerSecond, &Seconds) * FramesPerSecond + 1);
   int s = int(Seconds);
   int m = s / 60 % 60;
   int h = s / 3600;
   s %= 60;
-  snprintf(buffer, sizeof(buffer), WithFrame ? "%d:%02d:%02d.%02d" : "%d:%02d:%02d", h, m, s, f);
-  return buffer;
+  return cString::sprintf(WithFrame ? "%s%d:%02d:%02d.%02d" : "%s%d:%02d:%02d", Sign, h, m, s, f);
 }
 
 int HMSFToIndex(const char *HMSF, double FramesPerSecond)
@@ -2174,3 +2220,39 @@ int ReadFrame(cUnbufferedFile *f, uchar *b, int Length, int Max)
      LOG_ERROR;
   return r;
 }
+
+// --- Recordings Sort Mode --------------------------------------------------
+
+eRecordingsSortMode RecordingsSortMode = rsmName;
+
+bool HasRecordingsSortMode(const char *Directory)
+{
+  return access(AddDirectory(Directory, SORTMODEFILE), R_OK) == 0;
+}
+
+void GetRecordingsSortMode(const char *Directory)
+{
+  if (FILE *f = fopen(AddDirectory(Directory, SORTMODEFILE), "r")) {
+     char buf[8];
+     if (fgets(buf, sizeof(buf), f))
+        RecordingsSortMode = eRecordingsSortMode(constrain(atoi(buf), 0, int(rsmTime)));
+     fclose(f);
+     }
+}
+
+void SetRecordingsSortMode(const char *Directory, eRecordingsSortMode SortMode)
+{
+  if (FILE *f = fopen(AddDirectory(Directory, SORTMODEFILE), "w")) {
+     fputs(cString::sprintf("%d\n", SortMode), f);
+     fclose(f);
+     }
+}
+
+void IncRecordingsSortMode(const char *Directory)
+{
+  GetRecordingsSortMode(Directory);
+  RecordingsSortMode = eRecordingsSortMode(int(RecordingsSortMode) + 1);
+  if (RecordingsSortMode > rsmTime)
+     RecordingsSortMode = eRecordingsSortMode(0);
+  SetRecordingsSortMode(Directory, RecordingsSortMode);
+}
