@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: recording.h 2.25 2011/08/21 13:10:39 kls Exp $
+ * $Id: recording.h 2.37 2012/09/17 08:53:23 kls Exp $
  */
 
 #ifndef __RECORDING_H
@@ -22,10 +22,7 @@
 #define TIMERMACRO_TITLE    "TITLE"
 #define TIMERMACRO_EPISODE  "EPISODE"
 
-#define __RECORDING_H_DEPRECATED_DIRECT_MEMBER_ACCESS // Code enclosed with this macro is deprecated and may be removed in a future version
-
 extern bool VfatFileSystem;
-extern bool DirOrderState;
 extern int InstanceId;
 
 void RemoveDeletedRecordings(void);
@@ -81,16 +78,13 @@ public:
   bool Write(void) const;
   };
 
-#define SORTRECORDINGSVERSNUM 3
-#define MAXSORTMODES 4
-
 class cRecording : public cListObject {
   friend class cRecordings;
 private:
   mutable int resume;
   mutable char *titleBuffer;
-  mutable char *sortBuffer[MAXSORTMODES];
-  mutable char lastDirsFirst[MAXSORTMODES];
+  mutable char *sortBufferName;
+  mutable char *sortBufferTime;
   mutable char *fileName;
   mutable char *name;
   mutable int fileSizeMB;
@@ -98,6 +92,7 @@ private:
   int channel;
   int instanceId;
   bool isPesRecording;
+  mutable int isOnVideoDirectoryFileSystem; // -1 = unknown, 0 = no, 1 = yes
   double framesPerSecond;
   cRecordingInfo *info;
   cRecording(const cRecording&); // can't copy cRecording
@@ -105,9 +100,6 @@ private:
   static char *StripEpisodeName(char *s);
   char *SortName(void) const;
   int GetResume(void) const;
-#ifdef __RECORDING_H_DEPRECATED_DIRECT_MEMBER_ACCESS
-public:
-#endif
   time_t start;
   int priority;
   int lifetime;
@@ -123,9 +115,10 @@ public:
   virtual int Compare(const cListObject &ListObject) const;
   const char *Name(void) const { return name; }
   const char *FileName(void) const;
-  const char *Title(char Delimiter = ' ', bool NewIndicator = false, int Level = -1, bool Original = true) const;
+  const char *Title(char Delimiter = ' ', bool NewIndicator = false, int Level = -1) const;
   const cRecordingInfo *Info(void) const { return info; }
   const char *PrefixFileName(char Prefix);
+  const char *UpdateFileName(const char *FileName);
   int HierarchyLevels(void) const;
   void ResetResume(void) const;
   double FramesPerSecond(void) const { return framesPerSecond; }
@@ -134,9 +127,13 @@ public:
        ///< If the number of frames is unknown, -1 will be returned.
   int LengthInSeconds(void) const;
        ///< Returns the length (in seconds) of this recording, or -1 in case of error.
+  int FileSizeMB(void) const;
+       ///< Returns the total file size of this recording (in MB), or -1 if the file
+       ///< size is unknown.
   bool IsNew(void) const { return GetResume() <= 0; }
   bool IsEdited(void) const;
   bool IsPesRecording(void) const { return isPesRecording; }
+  bool IsOnVideoDirectoryFileSystem(void) const;
   void ReadInfo(void);
   bool WriteInfo(void);
   void SetStartTime(time_t Start);
@@ -157,9 +154,6 @@ public:
        ///< Changes the file name so that it will be visible in the "Recordings" menu again and
        ///< not processed by cRemoveDeletedRecordingsThread.
        ///< Returns false in case of error
-  bool Rename(const char *newName);
-       ///< Changes the file name
-       ///< Returns false in case of error
   };
 
 class cRecordings : public cList<cRecording>, public cThread {
@@ -168,7 +162,6 @@ private:
   bool deleted;
   time_t lastUpdate;
   int state;
-  int SortOrder;
   const char *UpdateFileName(void);
   void Refresh(bool Foreground = false);
   void ScanVideoDir(const char *DirName, bool Foreground = false, int LinkLevel = 0);
@@ -185,7 +178,7 @@ public:
        ///< Triggers an update of the list of recordings, which will run
        ///< as a separate thread if Wait is false. If Wait is true, the
        ///< function returns only after the update has completed.
-       ///< Returns true if Wait is true and there is anyting in the list
+       ///< Returns true if Wait is true and there is anything in the list
        ///< of recordings, false otherwise.
   void TouchUpdate(void);
        ///< Touches the '.update' file in the video directory, so that other
@@ -197,11 +190,12 @@ public:
   void ResetResume(const char *ResumeFileName = NULL);
   cRecording *GetByName(const char *FileName);
   void AddByName(const char *FileName, bool TriggerUpdate = true);
-  void DelByName(const char *FileName);
+  void DelByName(const char *FileName, bool RemoveRecording = true);
   void UpdateByName(const char *FileName);
-  int TotalFileSizeMB(void); ///< Only for deleted recordings!
-  void ToggleSortOrder(void) { SortOrder *= -1; }
-  const int GetSortOrder(void) { return SortOrder; }
+  int TotalFileSizeMB(void);
+  double MBperMinute(void);
+       ///< Returns the average data rate (in MB/min) of all recordings, or -1 if
+       ///< this value is unknown.
   };
 
 extern cRecordings Recordings;
@@ -213,9 +207,6 @@ class cMark : public cListObject {
   friend class cMarks; // for sorting
 private:
   double framesPerSecond;
-#ifdef __RECORDING_H_DEPRECATED_DIRECT_MEMBER_ACCESS
-public:
-#endif
   int position;
   cString comment;
 public:
@@ -250,13 +241,14 @@ public:
 #define RUC_BEFORERECORDING "before"
 #define RUC_AFTERRECORDING  "after"
 #define RUC_EDITEDRECORDING "edited"
+#define RUC_DELETERECORDING "deleted"
 
 class cRecordingUserCommand {
 private:
   static const char *command;
 public:
   static void SetCommand(const char *Command) { command = Command; }
-  static void InvokeCommand(const char *State, const char *RecordingFileName);
+  static void InvokeCommand(const char *State, const char *RecordingFileName, const char *SourceFileName = NULL);
   };
 
 // The maximum size of a single frame (up to HDTV 1920x1080):
@@ -293,17 +285,21 @@ private:
   cResumeFile resumeFile;
   cIndexFileGenerator *indexFileGenerator;
   cMutex mutex;
-  static cString IndexFileName(const char *FileName, bool IsPesRecording);
+  static cMutex indexListMutex;
+  static cVector<const cIndexFile *> indexList;
+  static void AddToIndexList(const cIndexFile *IndexFile);
+  static void RemoveFromIndexList(const cIndexFile *IndexFile);
+  static bool IsInIndexList(const cIndexFile *IndexFile);
   void ConvertFromPes(tIndexTs *IndexTs, int Count);
   void ConvertToPes(tIndexTs *IndexTs, int Count);
   bool CatchUp(int Index = -1);
 public:
-  cIndexFile(const char *FileName, bool Record, bool IsPesRecording = false);
+  cIndexFile(const char *FileName, bool Record, bool IsPesRecording = false, bool PauseLive = false);
   ~cIndexFile();
   bool Ok(void) { return index != NULL; }
   bool Write(bool Independent, uint16_t FileNumber, off_t FileOffset);
   bool Get(int Index, uint16_t *FileNumber, off_t *FileOffset, bool *Independent = NULL, int *Length = NULL);
-  int GetNextIFrame(int Index, bool Forward, uint16_t *FileNumber = NULL, off_t *FileOffset = NULL, int *Length = NULL, bool StayOffEnd = false);
+  int GetNextIFrame(int Index, bool Forward, uint16_t *FileNumber = NULL, off_t *FileOffset = NULL, int *Length = NULL);
   int Get(uint16_t FileNumber, off_t FileOffset);
   int Last(void) { CatchUp(); return last; }
   int GetResume(void) { return resumeFile.Read(); }
@@ -311,8 +307,9 @@ public:
   bool IsStillRecording(void);
   void Delete(void);
   static int GetLength(const char *FileName, bool IsPesRecording = false);
-       ///< Calculates the recording length (numer of frames) without actually reading the index file.
+       ///< Calculates the recording length (number of frames) without actually reading the index file.
        ///< Returns -1 in case of error.
+  static cString IndexFileName(const char *FileName, bool IsPesRecording);
   };
 
 class cFileName {
@@ -354,4 +351,11 @@ char *ExchangeChars(char *s, bool ToFileSystem);
 
 bool GenerateIndex(const char *FileName);
 
+enum eRecordingsSortMode { rsmName, rsmTime };
+extern eRecordingsSortMode RecordingsSortMode;
+bool HasRecordingsSortMode(const char *Directory);
+void GetRecordingsSortMode(const char *Directory);
+void SetRecordingsSortMode(const char *Directory, eRecordingsSortMode SortMode);
+void IncRecordingsSortMode(const char *Directory);
+
 #endif //__RECORDING_H
