@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: remux.c 2.62 2011/09/04 13:09:06 kls Exp $
+ * $Id: remux.c 2.67 2012/09/19 10:28:42 kls Exp $
  */
 
 #include "remux.h"
@@ -545,7 +545,7 @@ void cPatPmtParser::ParsePmt(const uchar *Data, int Length)
                       break;
            case 0x03: // STREAMTYPE_11172_AUDIO
            case 0x04: // STREAMTYPE_13818_AUDIO
-           case 0x0F: // ISO/IEC 13818-7 Audio with ADTS transport sytax
+           case 0x0F: // ISO/IEC 13818-7 Audio with ADTS transport syntax
            case 0x11: // ISO/IEC 14496-3 Audio with LATM transport syntax
                       {
                       if (NumApids < MAXAPIDS) {
@@ -675,6 +675,34 @@ void cPatPmtParser::ParsePmt(const uchar *Data, int Length)
                          }
                       }
                       break;
+           case 0x81: // STREAMTYPE_USER_PRIVATE
+                      {
+                      dbgpatpmt(" AC3");
+                      char lang[MAXLANGCODE1] = { 0 };
+                      SI::Descriptor *d;
+                      for (SI::Loop::Iterator it; (d = stream.streamDescriptors.getNext(it)); ) {
+                          switch (d->getDescriptorTag()) {
+                            case SI::ISO639LanguageDescriptorTag: {
+                                 SI::ISO639LanguageDescriptor *ld = (SI::ISO639LanguageDescriptor *)d;
+                                 dbgpatpmt(" '%s'", ld->languageCode);
+                                 strn0cpy(lang, I18nNormalizeLanguageCode(ld->languageCode), MAXLANGCODE1);
+                                 }
+                                 break;
+                            default: ;
+                            }
+                         delete d;
+                         }
+                      if (NumDpids < MAXDPIDS) {
+                         dpids[NumDpids] = stream.getPid();
+                         dtypes[NumDpids] = SI::AC3DescriptorTag;
+                         strn0cpy(dlangs[NumDpids], lang, sizeof(dlangs[NumDpids]));
+                         if (updatePrimaryDevice && Setup.UseDolbyDigital)
+                            cDevice::PrimaryDevice()->SetAvailableTrack(ttDolby, NumDpids, stream.getPid(), lang);
+                         NumDpids++;
+                         dpids[NumDpids]= 0;
+                         }
+                      }
+                      break;
            default: ;
            }
          dbgpatpmt("\n");
@@ -754,6 +782,10 @@ const uchar *cTsToPes::GetPes(int &Length)
         uchar *p = data + offset - 6;
         if (p != data) {
            p -= 3;
+           if (p < data) {
+              Reset();
+              return NULL;
+              }
            memmove(p, data, 4);
            }
         int l = min(length - offset, MAXPESLENGTH);
@@ -892,7 +924,8 @@ int cFrameDetector::SkipPackets(const uchar *&Data, int &Length, int &Processed,
 
 int cFrameDetector::Analyze(const uchar *Data, int Length)
 {
-  int SeenPayloadStart = false;
+  bool SeenPayloadStart = false;
+  bool SeenAccessUnitDelimiter = false;
   int Processed = 0;
   newFrame = independentFrame = false;
   while (Length >= TS_SIZE) {
@@ -1019,12 +1052,16 @@ int cFrameDetector::Analyze(const uchar *Data, int Length)
                                scanner = EMPTY_SCANNER;
                                if (synced && !SeenPayloadStart && Processed)
                                   return Processed; // flush everything before this new frame
+                               SeenAccessUnitDelimiter = true;
+                               }
+                            else if (SeenAccessUnitDelimiter && scanner == 0x00000001) { // NALU start
+                               SeenAccessUnitDelimiter = false;
                                int FrameTypeOffset = i + 1;
                                if (FrameTypeOffset >= TS_SIZE) // the byte to check is in the next TS packet
                                   i = SkipPackets(Data, Length, Processed, FrameTypeOffset);
                                newFrame = true;
-                               uchar FrameType = Data[FrameTypeOffset];
-                               independentFrame = FrameType == 0x10;
+                               uchar FrameType = Data[FrameTypeOffset] & 0x1F;
+                               independentFrame = FrameType == 0x07;
                                if (synced) {
                                   if (framesPerPayloadUnit < 0) {
                                      payloadUnitOfFrame = (payloadUnitOfFrame + 1) % -framesPerPayloadUnit;
