@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: videodir.c 2.0 2008/02/16 13:00:03 kls Exp $
+ * $Id: videodir.c 2.4 2012/09/30 12:06:33 kls Exp $
  */
 
 #include "videodir.h"
@@ -24,6 +24,11 @@
 
 const char *VideoDirectory = VIDEODIR;
 
+void SetVideoDirectory(const char *Directory)
+{
+  VideoDirectory = strdup(Directory);
+}
+
 class cVideoDirectory {
 private:
   char *name, *stored, *adjusted;
@@ -166,143 +171,6 @@ bool RenameVideoFile(const char *OldName, const char *NewName)
   return true;
 }
 
-bool MoveVideoFile(const char *SourceName, const char *TargetName, bool MakeCopy)
-{
-  const char* delim = "/";
-  cString source = SourceName;
-  cString target = TargetName;
-
-  // add missing directory delimiters
-  if (!endswith(*source, delim))
-     source = cString::sprintf("%s%s", *source, delim);
-  if (!endswith(*target, delim))
-     target = cString::sprintf("%s%s", *target, delim);
-
-  if (strcmp(*source, *target)) {
-     // validate target directory
-     if (strstr(*target, *source)) {
-        esyslog("ERROR: cannot move directory under its' sub-directory!\n");
-        return false;
-        }
-
-     RemoveFileOrDir(*target);
-     if (!MakeDirs(*target, true)) {
-        esyslog("ERROR: cannot create directory %s", *target);
-        return false;
-        }
-
-     if (!MakeCopy && EntriesOnSameFileSystem(*source, *target)) {
-        if (rename(*source, *target) == -1) {
-           LOG_ERROR_STR(*source);
-           return false;
-           }
-        }
-     else {
-        int required = DirSizeMB(*source);
-        int available = FreeDiskSpaceMB(*target);
-
-        // validate free space
-        if (required < available) {
-           cReadDir d(*source);
-           struct dirent *e;
-           bool success = true;
-
-           // allocate copying buffer
-           const int len = 1024 * 1024;
-           char *buffer = MALLOC(char, len);
-           if (!buffer) {
-              esyslog("ERROR: cannot allocate renaming buffer");
-              return false;
-              }
-
-           // loop through all files, but skip all sub-directories
-           while ((e = d.Next()) != NULL) {
-                 // skip generic entries
-                 if (strcmp(e->d_name, ".") && strcmp(e->d_name, "..") && strcmp(e->d_name, "lost+found")) {
-                    cString sourceFile = cString::sprintf("%s%s", *source, e->d_name);
-                    cString targetFile = cString::sprintf("%s%s", *target, e->d_name);
-
-                    // copy only regular files
-                    struct stat sts;
-                    if (!stat(*sourceFile, &sts) && S_ISREG(sts.st_mode)) {
-                       int r = -1, w = -1;
-                       cUnbufferedFile *inputFile = cUnbufferedFile::Create(*sourceFile, O_RDONLY | O_LARGEFILE);
-                       cUnbufferedFile *outputFile = cUnbufferedFile::Create(*targetFile, O_RDWR | O_CREAT | O_LARGEFILE);
-
-                       // validate files
-                       if (!inputFile || !outputFile) {
-                          esyslog("ERROR: cannot open file %s or %s", *sourceFile, *targetFile);
-                          success = false;
-                          break;
-                          }
-
-                       // do actual copy
-                       dsyslog("copying %s to %s", *sourceFile, *targetFile);
-                       do {
-                         r = inputFile->Read(buffer, len);
-                         if (r > 0)
-                            w = outputFile->Write(buffer, r);
-                         else
-                            w = 0;
-                       } while (r > 0 && w > 0);
-                       DELETENULL(inputFile);
-                       DELETENULL(outputFile);
-
-                       // validate result
-                       if (r < 0 || w < 0) {
-                          success = false;
-                          break;
-                          }
-                       }
-                    }
-                 }
-
-           // release allocated buffer
-           free(buffer);
-
-           // delete all created target files and directories
-           if (!success) {
-              target = StripLastDirectory(*target);
-              if (!RemoveFileOrDir(*target, true))
-                 esyslog("ERROR: cannot remove target %s", *target);
-              target = StripLastDirectory(*target);
-              if (!RemoveEmptyDirectories(*target, true))
-                 esyslog("ERROR: cannot remove target directory %s", *target);
-              esyslog("ERROR: copying failed");
-              return false;
-              }
-           else if (!MakeCopy && !RemoveFileOrDir(*source, true)) { // delete source files
-              esyslog("ERROR: cannot remove source directory %s", *source);
-              return false;
-              }
-
-           // delete all empty source directories
-           if (!MakeCopy) {
-              source = StripLastDirectory(*source);
-              while (!strcmp(*source, VideoDirectory)) {
-                    source = StripLastDirectory(*source);
-                    if (!RemoveEmptyDirectories(*source, true))
-                        break;
-                    }
-              }
-           }
-        else {
-           esyslog("ERROR: %sing requires %dMB - only %dMB available", MakeCopy ? "copy" : "mov", required, available);
-           // delete all created empty target directories
-           target = StripLastDirectory(*target);
-           while (!strcmp(*target, VideoDirectory)) {
-                 target = StripLastDirectory(*target);
-                 if (!RemoveEmptyDirectories(*target, true))
-                     break;
-                 }
-           return false;
-           }
-        }
-     }
-
-  return true;
-}
-
 bool RemoveVideoFile(const char *FileName)
 {
   return RemoveFileOrDir(FileName, true);
@@ -478,11 +346,27 @@ cString PrefixVideoFileName(const char *FileName, char Prefix)
   return NULL;
 }
 
-void RemoveEmptyVideoDirectories(void)
+cString NewVideoFileName(const char *FileName, const char *NewDirName)
+{
+  char *NewDir = ExchangeChars(strdup(NewDirName), true);
+  if (NewDir) {
+     const char *p = FileName + strlen(FileName); // p points at the terminating 0
+     while (p-- > FileName) {
+           if (*p == '/')
+              break;
+           }
+     cString NewName = cString::sprintf("%s/%s%s", VideoDirectory, NewDir, p);
+     free(NewDir);
+     return NewName;
+     }
+  return NULL;
+}
+
+void RemoveEmptyVideoDirectories(const char *IgnoreFiles[])
 {
   cVideoDirectory Dir;
   do {
-     RemoveEmptyDirectories(Dir.Name());
+     RemoveEmptyDirectories(Dir.Name(), false, IgnoreFiles);
      } while (Dir.Next());
 }
 
@@ -495,3 +379,43 @@ bool IsOnVideoDirectoryFileSystem(const char *FileName)
      } while (Dir.Next());
   return false;
 }
+
+// --- cVideoDiskUsage -------------------------------------------------------
+
+#define DISKSPACECHEK     5 // seconds between disk space checks
+#define MB_PER_MINUTE 25.75 // this is just an estimate!
+
+int cVideoDiskUsage::state = 0;
+time_t cVideoDiskUsage::lastChecked = 0;
+int cVideoDiskUsage::usedPercent = 0;
+int cVideoDiskUsage::freeMB = 0;
+int cVideoDiskUsage::freeMinutes = 0;
+
+bool cVideoDiskUsage::HasChanged(int &State)
+{
+  if (time(NULL) - lastChecked > DISKSPACECHEK) {
+     int FreeMB;
+     int UsedPercent = VideoDiskSpace(&FreeMB);
+     if (FreeMB != freeMB) {
+        usedPercent = UsedPercent;
+        freeMB = FreeMB;
+        double MBperMinute = Recordings.MBperMinute();
+        if (MBperMinute <= 0)
+           MBperMinute = MB_PER_MINUTE;
+        freeMinutes = int(double(FreeMB) / MBperMinute);
+        state++;
+        }
+     lastChecked = time(NULL);
+     }
+  if (State != state) {
+     State = state;
+     return true;
+     }
+  return false;
+}
+
+cString cVideoDiskUsage::String(void)
+{
+  HasChanged(state);
+  return cString::sprintf("%s %d%%  -  %2d:%02d %s", tr("Disk"), usedPercent, freeMinutes / 60, freeMinutes % 60, tr("free"));
+}
